'''
Task Coach - Your friendly task manager
Copyright (C) 2011 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
from taskcoachlib.i18n import _


class KeychainPasswordWidget(wx.Dialog):
    def __init__(self, domain, username, *args, **kwargs):
        super(KeychainPasswordWidget, self).__init__(*args, **kwargs)

        self.domain = domain.encode('UTF-8')
        self.username = username.encode('UTF-8')

        pnl = wx.Panel(self, wx.ID_ANY)
        hsz = wx.BoxSizer(wx.HORIZONTAL)
        hsz.Add(wx.StaticText(pnl, wx.ID_ANY, _('Password:')), 0, wx.ALL, 3)

        from taskcoachlib.thirdparty.keyring import get_password
        password = get_password(self.domain, self.username)
        self.password = (password or '').decode('UTF-8')
        self.passwordField = wx.TextCtrl(pnl, wx.ID_ANY, self.password, style=wx.TE_PASSWORD)
        hsz.Add(self.passwordField, 1, wx.ALL, 3)

        vsz = wx.BoxSizer(wx.VERTICAL)
        vsz.Add(hsz, 0, wx.ALL|wx.EXPAND, 3)
        self.keepInKeychain = wx.CheckBox(pnl, wx.ID_ANY, _('Store in keychain'))
        self.keepInKeychain.SetValue(bool(password))
        vsz.Add(self.keepInKeychain, 0, wx.ALL|wx.EXPAND, 3)

        hsz = wx.BoxSizer(wx.HORIZONTAL)
        btnOK = wx.Button(pnl, wx.ID_ANY, _('OK'))
        hsz.Add(btnOK, 0, wx.ALL, 3)
        btnCancel = wx.Button(pnl, wx.ID_ANY, _('Cancel'))
        hsz.Add(btnCancel, 0, wx.ALL, 3)
        vsz.Add(hsz, 0, wx.ALL|wx.ALIGN_CENTRE, 3)

        pnl.SetSizer(vsz)

        sz = wx.BoxSizer(wx.HORIZONTAL)
        sz.Add(pnl, 1, wx.EXPAND|wx.ALL, 3)
        self.SetSizer(sz)
        self.Fit()

        wx.EVT_BUTTON(btnOK, wx.ID_ANY, self.OnOK)
        wx.EVT_BUTTON(btnCancel, wx.ID_ANY, self.OnCancel)

        self.SetDefaultItem(btnOK)
        wx.CallAfter(self.RequestUserAttention)

    def OnOK(self, event):
        self.password = self.passwordField.GetValue()
        from taskcoachlib.thirdparty.keyring import set_password
        if self.keepInKeychain.GetValue():
            set_password(self.domain, self.username, self.password.encode('UTF-8'))
        else:
            set_password(self.domain, self.username, '')
        self.EndModal(wx.ID_OK)

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL)


_PASSWORDCACHE = None

def _GetCachedPassword(domain, username, reset):
    global _PASSWORDCACHE

    if _PASSWORDCACHE is None:
        import StringIO, traceback
        bf = StringIO.StringIO()
        traceback.print_exc(file=bf)
        wx.MessageBox(_('There was a problem trying to find out your system\'s keychain.\nPlease file a bug report (see the Help menu) and attach a screenshot of this message.\nError was:\n\n%s') % bf.getvalue(), _('Error'), wx.OK)
        _PASSWORDCACHE = dict()
    if (domain, username) in _PASSWORDCACHE and reset:
        del _PASSWORDCACHE[(domain, username)]
    if (domain, username) not in _PASSWORDCACHE:
        pwd = wx.GetPasswordFromUser(_('Please enter your password.'), domain)
        if not pwd:
            return None
        _PASSWORDCACHE[(domain, username)] = pwd
    return _PASSWORDCACHE[(domain, username)]

def GetPassword(domain, username, reset=False):
    try:
        from taskcoachlib.thirdparty.keyring import set_password, get_password
    except:
        # Keychain unavailable.
        return _GetCachedPassword(domain, username, reset)

    try:
        if reset:
            set_password(domain.encode('UTF-8'), username.encode('UTF-8'), '')
        else:
            pwd = get_password(domain.encode('UTF-8'), username.encode('UTF-8'))
            if pwd:
                return pwd.decode('UTF-8')
    except ImportError:
        # Bug seen on Ubuntu 13.10: secretstorage cannot import ._gi
        return _GetCachedPassword(domain, username, reset)

    dlg = KeychainPasswordWidget(domain, username, None, wx.ID_ANY, _('Please enter your password'), style=wx.DEFAULT_DIALOG_STYLE|wx.STAY_ON_TOP)
    try:
        dlg.CentreOnScreen()
        if dlg.ShowModal() == wx.ID_OK:
            return dlg.password
    finally:
        dlg.Destroy()



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import inspect
from taskcoachlib.syncml import basesource
from taskcoachlib.domain.task import Task
from taskcoachlib.domain.category import Category
from taskcoachlib.persistence.icalendar import ical


class TaskSource(basesource.BaseSource):
    def __init__(self, callback, taskList, categoryList, *args, **kwargs):
        super(TaskSource, self).__init__(callback, taskList, *args, **kwargs)
        self.categoryList = categoryList

    def updateItemProperties(self, item, task):
        item.data = 'BEGIN:VCALENDAR\r\nVERSION: 1.0\r\n' + \
                    ical.VCalFromTask(task, doFold=False).encode('UTF-8') + \
                    'END:VCALENDAR'
        item.dataType = 'text/calendar'

    def _parseObject(self, item):
        parser = ical.VCalendarParser()
        parser.parse(map(lambda x: x.rstrip('\r'), item.data.split('\n')))

        categories = parser.tasks[0].pop('categories', [])

        kwargs = dict([(k, v) for k, v in parser.tasks[0].items() if k in inspect.getargspec(Task.__init__)[0]])
        task = Task(**kwargs)

        for category in categories:
            categoryObject = self.categoryList.findCategoryByName(category)
            if categoryObject is None:
                categoryObject = Category(category)
                self.categoryList.extend([categoryObject])
            task.addCategory(categoryObject)

        return task

    def doAddItem(self, task):
        for category in task.categories():
            category.addCategorizable(task)

        return 201

    def doUpdateItem(self, task, local):
        local.setPlannedStartDateTime(task.plannedStartDateTime())
        local.setDueDateTime(task.dueDateTime())
        local.setDescription(task.description())
        local.setSubject(task.subject())
        local.setPriority(task.priority())
        local.setCompletionDateTime(task.completionDateTime())

        for category in local.categories():
            category.removeCategorizable(local)

        local.setCategories(task.categories())

        for category in local.categories():
            category.addCategorizable(local)

        super(TaskSource, self).doUpdateItem(task, local)

        return 200 # FIXME



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>
Copyright (C) 2008 Rob McMullen <rob.mcmullen@gmail.com>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib import persistence, patterns, operating_system
from taskcoachlib.i18n import _
import wx


# Prepare for printing. On Jolicloud, printing crashes unless we do this:
if operating_system.isGTK():
    try:
        import gtk  # pylint: disable=F0401
        gtk.remove_log_handlers()
    except ImportError:
        pass


class PrinterSettings(object):
    __metaclass__ = patterns.Singleton

    edges = ('top', 'left', 'bottom', 'right')

    def __init__(self, settings):
        self.settings = settings
        self.printData = wx.PrintData()
        self.pageSetupData = wx.PageSetupDialogData(self.printData)
        self.__initialize_from_settings()

    def updatePageSetupData(self, data):
        self.pageSetupData = wx.PageSetupDialogData(data)
        self.__update_print_data(data.GetPrintData())
        self.__save_to_settings()

    def __update_print_data(self, printData):
        self.printData = wx.PrintData(printData)
        self.pageSetupData.SetPrintData(self.printData)
 
    def __getattr__(self, attr):
        try:
            return getattr(self.pageSetupData, attr)
        except AttributeError:
            return getattr(self.printData, attr)

    def __initialize_from_settings(self):
        ''' Load the printer settings from the user settings. '''
        margin = dict()
        for edge in self.edges:
            margin[edge] = self.__get_setting('margin_' + edge)
        top_left = wx.Point(margin['left'], margin['top'])
        bottom_right = wx.Point(margin['right'], margin['bottom'])
        self.SetMarginTopLeft(top_left)
        self.SetMarginBottomRight(bottom_right)
        self.SetPaperId(self.__get_setting('paper_id'))
        self.SetOrientation(self.__get_setting('orientation'))

    def __save_to_settings(self):
        ''' Save the printer settings to the user settings. '''
        margin = dict()
        margin['left'], margin['top'] = self.GetMarginTopLeft()  
        margin['right'], margin['bottom'] = self.GetMarginBottomRight()  
        for edge in self.edges:
            self.__set_setting('margin_'+edge, margin[edge])
        self.__set_setting('paper_id', self.GetPaperId())
        self.__set_setting('orientation', self.GetOrientation())

    def __get_setting(self, option):
        return self.settings.getint('printer', option)

    def __set_setting(self, option, value):
        self.settings.set('printer', option, str(value))


class HTMLPrintout(wx.html.HtmlPrintout):
    def __init__(self, html_text, settings):
        super(HTMLPrintout, self).__init__()
        self.SetHtmlText(html_text)
        self.SetFooter(_('Page') + ' @PAGENUM@/@PAGESCNT@', wx.html.PAGE_ALL)
        self.SetFonts('Arial', 'Courier')
        printer_settings = PrinterSettings(settings)
        left, top = printer_settings.pageSetupData.GetMarginTopLeft()
        right, bottom = printer_settings.pageSetupData.GetMarginBottomRight()
        self.SetMargins(top, bottom, left, right)

                
class DCPrintout(wx.Printout):
    def __init__(self, widget):
        self.widget = widget
        super(DCPrintout, self).__init__()
        
    def OnPrintPage(self, page):  # pylint: disable=W0613
        self.widget.Draw(self.GetDC())
        
    def GetPageInfo(self):  # pylint: disable=W0221
        return (1, 1, 1, 1)

        
def Printout(viewer, settings, printSelectionOnly=False, 
             twoPrintouts=False):
    widget = viewer.getWidget()
    if hasattr(widget, 'GetPrintout'):
        _printout = widget.GetPrintout
    elif hasattr(widget, 'Draw'):
        def _printout(settings):
            return DCPrintout(widget)
    else:
        html_text = persistence.viewer2html(viewer, settings, 
                                           selectionOnly=printSelectionOnly)[0]
        def _printout(settings):
            return HTMLPrintout(html_text, settings)
    result = _printout(PrinterSettings(settings))
    if twoPrintouts:
        result = (result, _printout(PrinterSettings(settings)))
    return result



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib.i18n import _
from taskcoachlib import operating_system
import wx, subprocess, threading, time


class MailAppInfoLoader(wx.ProgressDialog):
    def __init__(self, parent):
        super(MailAppInfoLoader, self).__init__(_('Reading mail info...'),
            _('Reading mail information. Please wait.'), parent=parent,
            style=wx.PD_SMOOTH|wx.PD_ELAPSED_TIME|wx.PD_CAN_SKIP)

        self.__title = None
        thread = threading.Thread(target=self.__run)
        thread.start()

    def __run(self):
        script = '''
tell application "Mail"
    set theMessages to selection
    subject of beginning of theMessages
end tell
'''
        try:
            sp = subprocess.Popen('osascript', stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            out, err = sp.communicate(script)
            if not sp.returncode:
                self.__title = operating_system.decodeSystemString(out.strip())
                return
        except:
            pass

        self.__title = _('Mail.app message')

    def title(self):
        return self.__title


def getSubjectOfMail(messageId): # pylint: disable=W0613
    """This should return the subject of the mail having the specified
    message-id. Unfortunately, until I find an Applescript guru, it
    will only return the subject of the currently selected mail in
    Mail.app."""

    dlg = MailAppInfoLoader(None)
    dlg.ShowModal()
    try:
        while dlg.title() is None:
            time.sleep(0.2)
            unused, skip = dlg.Pulse()
            if skip:
                return _('Mail.app message')
        return dlg.title()
    finally:
        dlg.Destroy()



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import test
from taskcoachlib import gui, config
from taskcoachlib.domain import task, date


class DummyViewer(object):
    def __init__(self, presentation):
        self._presentation = presentation
        self._selection = []
        
    def presentation(self):
        return self._presentation
    
    def curselection(self):
        return self._selection
    
    def nrOfVisibleTasks(self):
        return len(self._presentation)
    

class TaskViewerStatusMessagesTest(test.TestCase):
    def setUp(self):
        super(TaskViewerStatusMessagesTest, self).setUp()
        task.Task.settings = config.Settings(load=False)
        self.taskList = task.filter.ViewFilter(task.TaskList())
        self.task = task.Task('Task')
        self.viewer = DummyViewer(self.taskList)
        self.status = gui.viewer.task.TaskViewerStatusMessages(self.viewer)
        self.template1 = 'Tasks: %d selected, %d visible, %d total'
        self.template2 = 'Status: %d overdue, %d late, %d inactive, %d completed'
    
    # Helper methods
        
    def assertMessages(self, selected=0, visible=0, total=0, overdue=0, 
                       late=0, inactive=0, completed=0):
        message1 = self.template1%(selected, visible, total)     
        message2 = self.template2%(overdue, late, inactive, completed)
        self.assertEqual((message1, message2), self.status())
        
    def addActiveTask(self):
        self.task.setActualStartDateTime(date.Now())
        self.taskList.append(self.task)
        
    def addOverdueTask(self):
        self.task.setDueDateTime(date.Now() - date.ONE_HOUR)
        self.taskList.append(self.task)

    def addInactiveTask(self):
        self.taskList.append(self.task)
        
    def addCompletedTask(self):
        self.task.setCompletionDateTime(date.Now())
        self.taskList.append(self.task)
        
    def addLateTask(self):
        self.task.setPlannedStartDateTime(date.Yesterday())
        self.taskList.append(self.task)
        
    def removeTask(self):
        self.taskList.remove(self.task)
        
    def markTaskCompleted(self):
        self.task.setCompletionDateTime(date.Now())

    def markTaskUncompleted(self):
        self.task.setCompletionDateTime(date.DateTime())
        
    def makeTaskActive(self):
        self.task.setActualStartDateTime(date.Now())
        
    def makeTaskInactive(self):
        self.task.setActualStartDateTime(date.DateTime())
        
    def selectTask(self):
        self.viewer._selection = [self.task]
        
    def hideCompletedTasks(self):
        self.taskList.hideTaskStatus(task.status.completed)
        
    def showCompletedTasks(self):
        self.taskList.hideTaskStatus(task.status.completed, False)
        
    # Tests
    
    def testDefaultMessages(self):
        self.assertMessages()
        
    def testAddActiveTask(self):
        self.addActiveTask()
        self.assertMessages(visible=1, total=1)

    def testAddInactiveTask(self):
        self.addInactiveTask()
        self.assertMessages(visible=1, total=1, inactive=1)
        
    def testAddOverdueTask(self):
        self.addOverdueTask()
        self.assertMessages(visible=1, total=1, inactive=0, overdue=1)

    def testAddCompletedTask(self):
        self.addCompletedTask()
        self.assertMessages(visible=1, total=1, completed=1)
        
    def testAddLateTask(self):
        self.addLateTask()
        self.assertMessages(visible=1, late=1, total=1)
        
    def testRemoveActiveTask(self):
        self.addActiveTask()
        self.removeTask()
        self.assertMessages()

    def testRemoveInactiveTask(self):
        self.addInactiveTask()
        self.removeTask()
        self.assertMessages()

    def testRemoveOverdueTask(self):
        self.addOverdueTask()
        self.removeTask()
        self.assertMessages()
                       
    def testRemoveCompletedTask(self):
        self.addCompletedTask()
        self.removeTask()
        self.assertMessages()
        
    def testMarkInactiveTaskCompleted(self):
        self.addInactiveTask()
        self.markTaskCompleted()        
        self.assertMessages(visible=1, total=1, completed=1)

    def testMarkActiveTaskCompleted(self):
        self.addCompletedTask()
        self.markTaskCompleted()
        self.assertMessages(visible=1, total=1, completed=1)
        
    def testMarkCompletedTaskUncompleted(self):
        self.addCompletedTask()
        self.markTaskUncompleted()
        self.assertMessages(visible=1, total=1, inactive=1)
                
    def testMakeInactiveTaskActive(self):
        self.addInactiveTask()
        self.makeTaskActive()    
        self.assertMessages(visible=1, total=1)
        
    def testMakeActiveTaskInactive(self):
        self.addActiveTask()
        self.makeTaskInactive()    
        self.assertMessages(visible=1, total=1, inactive=1)
        
    def testMakeCompletedTaskInactive(self):
        self.addActiveTask()
        self.markTaskCompleted()
        self.makeTaskInactive()
        # Completed tasks are never considered to be inactive:
        self.assertMessages(visible=1, total=1, completed=1)
        
    def testMakeCompletedTaskActive(self):
        self.addInactiveTask()
        self.markTaskCompleted()
        self.makeTaskActive()
        # Completed tasks are never considered to be inactive:
        self.assertMessages(visible=1, total=1, completed=1)

    def testTotalWhenHidingCompletedTasks(self):
        self.addCompletedTask()
        self.hideCompletedTasks()
        self.assertMessages(total=1, completed=1)
        
    def testTotalWhenShowingCompletedTasks(self):
        self.hideCompletedTasks()
        self.addCompletedTask()
        self.showCompletedTasks()
        self.assertMessages(visible=1, total=1, completed=1)
        
    def testTotalWhenHidingCompletedTasksWithActiveTask(self):
        self.taskList.append(task.Task(actualStartDateTime=date.Now()))
        self.addCompletedTask()
        self.hideCompletedTasks()
        self.assertMessages(visible=1, total=2, completed=1)
        
    def testSelectedActiveTask(self):
        self.addActiveTask()
        self.selectTask()
        self.assertMessages(selected=1, visible=1, total=1)

    def testSelectedInactiveTask(self):
        self.addInactiveTask()
        self.selectTask()
        self.assertMessages(selected=1, visible=1, total=1, inactive=1)

    def testSelectedCompletedTask(self):
        self.addCompletedTask()
        self.selectTask()
        self.assertMessages(selected=1, visible=1, total=1, completed=1)

    def testSelectedOverdueTask(self):
        self.addOverdueTask()
        self.selectTask()
        self.assertMessages(selected=1, visible=1, total=1, overdue=1, 
                            inactive=0)



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib import config, meta
from unittests import dummy
import StringIO
import test


class DeveloperMessageCheckerUnderTest(meta.DeveloperMessageChecker):
    def __init__(self, *args, **kwargs):
        self.message_file_contents = 'Message|http://a.b\n'
        kwargs['urlopen'] = self.urlopen
        kwargs['call_after'] = self.call_after
        super(DeveloperMessageCheckerUnderTest, self).__init__(*args, **kwargs)
        
    @staticmethod
    def call_after(function, *args, **kwargs):
        # Don't use wx.CallAfter in the tests
        return function(*args, **kwargs)
    
    def urlopen(self, url):  # pylint: disable=W0613
        return StringIO.StringIO(self.message_file_contents)
    

class DeveloperMessageCheckerTest(test.TestCase):
    def setUp(self):
        self.settings = config.Settings(load=False)
        self.checker = DeveloperMessageCheckerUnderTest(self.settings)
        
    def testDialogContainsMessage(self):
        dialog = self.checker.run(show=False)
        self.assertEqual('Message', dialog.current_message())

    def testDialogContainsURL(self):
        dialog = self.checker.run(show=False)
        self.assertEqual('http://a.b', dialog.current_url())

    def testNoMessage(self):
        self.checker.message_file_contents = ''
        self.assertEqual(None, self.checker.run(show=False))
 
    def testOnlyCommentInMessageFile(self):
        self.checker.message_file_contents = '# This is a comment | even ' \
                                             'with a pipe symbol in it'
        self.assertEqual(None, self.checker.run(show=False))
        
    def testCommentFollowedByEmptyLineInMessageFile(self):
        self.checker.message_file_contents = '# This is a comment\n\n'
        self.assertEqual(None, self.checker.run(show=False))
        
    def testCommentFollowedWithMessage(self):
        self.checker.message_file_contents = '# This is a comment\n' \
                                             'This is the message|url\n'
        dialog = self.checker.run(show=False)
        self.assertEqual('This is the message', dialog.current_message())
        
    def testDontShowSameMessageTwice(self):
        dialog = self.checker.run(show=False)
        self.assertEqual('Message', dialog.current_message())
        dialog.on_close(dummy.Event())
        self.assertEqual(None, self.checker.run(show=False))
 


'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from unittests import asserts
from CommandTestCase import CommandTestCase
from TaskCommandsTest import TaskCommandTestCase, CommandWithChildrenTestCase, \
    CommandWithEffortTestCase
from taskcoachlib import command
from taskcoachlib.domain import note


class CutCommandWithTasksTest(TaskCommandTestCase):
    def testCutTasks_WithoutSelection(self):
        self.cut()
        self.assertDoUndoRedo(lambda: self.assertTaskList([self.task1]))

    def testCutTasks_SpecificTask(self):
        self.taskList.append(self.task2)
        self.cut([self.task1])
        self.assertDoUndoRedo(lambda: (self.assertTaskList([self.task2]),
            self.assertEqual([self.task1], command.Clipboard().get()[0])),
            lambda: (self.assertTaskList([self.task1, self.task2]),
            self.failIf(command.Clipboard())))

    def testCutTasks_All(self):
        self.cut('all')
        self.assertDoUndoRedo(self.assertEmptyTaskList, 
            lambda: self.assertTaskList(self.originalList))
        
    def testCutTaskThatBelongsToCategory(self):
        self.category.addCategorizable(self.task1)
        self.task1.addCategory(self.category)
        self.cut('all')
        self.assertDoUndoRedo(lambda: self.failIf(self.category.categorizables()),
                              lambda: self.assertEqual(set([self.task1]), 
                                                       self.category.categorizables()))


class CutCommandWithTasksWithChildrenTest(CommandWithChildrenTestCase):
    def testCutParent(self):
        self.cut([self.parent])
        self.assertDoUndoRedo(lambda: self.assertTaskList([self.task1]),
            lambda: self.assertTaskList(self.originalList))

    def testCutChild(self):
        self.cut([self.child])
        self.assertDoUndoRedo(
            lambda: (self.assertTaskList([self.task1, self.child2, self.parent]),
            self.assertEqual([self.child2], self.parent.children())),
            lambda: (self.assertTaskList(self.originalList),
            self.failUnlessParentAndChild(self.parent, self.child)))

    def testCutParentAndChild(self):
        self.cut([self.child, self.parent])
        self.assertDoUndoRedo(lambda: self.assertTaskList([self.task1]),
            lambda: self.assertTaskList(self.originalList))


class CutCommandWithEffortTest(CommandWithEffortTestCase):
    def testCutEfforts_WithoutSelection(self):
        self.cut()
        self.assertDoUndoRedo(lambda: self.assertEffortList(self.originalEffortList))

    def testCutEfforts_Selection(self):
        self.cut([self.effort1])
        self.assertDoUndoRedo(lambda: self.assertEffortList([self.effort2]),
                              lambda: self.assertEffortList(self.originalEffortList))
                              
    def testCutEfforts_All(self):
        self.cut('all')
        self.assertDoUndoRedo(lambda: self.assertEffortList([]),
                              lambda: self.assertEffortList(self.originalEffortList))


class NoteCommandTestCase(CommandTestCase, asserts.Mixin):
    def setUp(self):
        super(NoteCommandTestCase, self).setUp()
        self.note1 = note.Note()
        self.note2 = note.Note()
        self.list = self.noteContainer = note.NoteContainer([self.note1, self.note2])
        self.original = note.NoteContainer([self.note1, self.note2])


class CutCommandWithNotes(NoteCommandTestCase):        
    def testCutNotes_WithoutSelection(self):
        self.cut()
        self.assertDoUndoRedo(lambda: self.assertNoteContainer(self.original))

    def testCutNotes_Selection(self):
        self.cut([self.note1])
        self.assertDoUndoRedo(lambda: self.assertNoteContainer(note.NoteContainer([self.note2])),
            lambda: self.assertNoteContainer(self.original))
        
    def testCutNotes_All(self):
        self.cut('all')
        self.assertDoUndoRedo(lambda: self.assertNoteContainer(note.NoteContainer()),
            lambda: self.assertNoteContainer(self.original))
        

class PasteCommandWithTasksTest(TaskCommandTestCase):
    def testPasteWithoutPreviousCut(self):
        self.paste()
        self.assertDoUndoRedo(lambda: self.assertTaskList(self.originalList))

    def testPaste(self):
        self.cut([self.task1])
        self.paste()
        self.assertDoUndoRedo(lambda: self.assertEqual(1, len(self.taskList)),
            self.assertEmptyTaskList)

    def testClipboardIsNotEmptyAfterPaste(self):
        self.cut([self.task1])
        self.paste()
        # pylint: disable=W0212
        self.assertDoUndoRedo(
            lambda: self.assertEqual(1, len(command.Clipboard()._contents)))


class PasteCommandWithNotesTest(NoteCommandTestCase):
    def testPasteWithoutPreviousCut(self):
        self.paste()
        self.assertDoUndoRedo(lambda: self.assertNoteContainer(self.original))
        
    def testPaste(self):
        self.cut([self.note1])
        self.paste()
        self.assertDoUndoRedo(lambda: self.assertEqual(2, len(self.original)),
            lambda: self.assertNoteContainer([self.note2]))
            

class PasteCommandWithEffortTest(CommandWithEffortTestCase):
    def testPasteWithoutPreviousCut(self):
        self.paste()
        self.assertDoUndoRedo(lambda: self.assertEffortList(self.originalEffortList))

    def testPaste(self):
        self.cut([self.effort1])
        self.paste()
        self.assertDoUndoRedo(lambda: self.assertEqual(2, len(self.originalEffortList)),
            lambda: self.assertEqualLists([self.effort2], self.effortList))

    def testClipboardIsNotEmptyAfterPaste(self):
        self.cut([self.effort1])
        self.paste()
        # pylint: disable=W0212
        self.assertDoUndoRedo(
            lambda: self.assertEqual([self.effort1], command.Clipboard()._contents))
        
        
class PasteCommandWithTasksWithChildrenTest(CommandWithChildrenTestCase):
    def testCutAndPasteChild(self):
        self.cut([self.child])
        self.paste()
        self.assertDoUndoRedo(lambda: self.assertEqual(5, len(self.taskList)),
            lambda: self.assertTaskList([self.task1, self.parent, self.child2]))

    def testCutAndPasteParentAndChild(self):
        self.cut([self.parent, self.child])
        self.paste()
        self.assertDoUndoRedo(lambda: self.assertEqual(5, len(self.taskList)),
            lambda: self.assertTaskList([self.task1]))

    def testCutAndPasteParentAndGrandChild(self):
        self.cut([self.parent, self.grandchild])
        self.paste()
        self.assertDoUndoRedo(lambda: (self.assertEqual(5, len(self.taskList)),
            self.failUnlessParentAndChild(self.parent, self.child)),
            lambda: self.assertTaskList([self.task1]))


class PasteIntoTaskCommandTest(CommandWithChildrenTestCase):
    def testPasteChild(self):
        self.cut([self.child])
        self.paste([self.task1])
        self.assertDoUndoRedo(
            lambda: (self.assertEqual(5, len(self.taskList)),
            self.assertEqual(1, len(self.task1.children())), 
            self.failUnlessParentAndChild(self.child, self.grandchild)),
            lambda: (self.assertEqual(1, len(self.parent.children())),
            self.assertEqual([], self.task1.children())))

    def testPasteExtraChild(self):
        self.cut([self.task1])
        self.paste([self.parent])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(3, len(self.parent.children())),
            lambda: (self.assertEqual(2, len(self.parent.children())), 
                     self.assertTaskList([self.parent, self.child, self.child2, 
                                          self.grandchild])))

    def testPasteChild_MarksNewParentAsNotCompleted(self):
        self.settings.setboolean('behavior', 
                                 'markparentcompletedwhenallchildrencompleted', 
                                 True)
        self.markCompleted([self.parent])
        self.cut([self.task1])
        self.paste([self.parent])
        self.assertDoUndoRedo(
            lambda: self.failIf(self.parent.completed()),
            lambda: self.failUnless(self.parent.completed()))

    def testPasteCompletedChild_DoesNotMarkParentAsNotCompleted(self):
        self.markCompleted([self.task1, self.parent])
        self.cut([self.task1])
        self.paste([self.parent])
        self.assertDoUndoRedo(
            lambda: self.failUnless(self.parent.completed()),
            lambda: self.failUnless(self.parent.completed()))


class PasteIntoTaskCommandWithEffortTest(CommandWithEffortTestCase):
    def testPaste(self):
        self.cut([self.effort1])
        self.paste([self.task2])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(2, len(self.task2.efforts())),
            lambda: self.assertEqual(1, len(self.task1.efforts())))

        
class CutAndPasteTasksIntegrationTest(TaskCommandTestCase):
    def testUndoCutAndPaste(self):
        self.cut([self.task1])
        self.paste()
        self.undo()
        self.undo()
        self.assertTaskList(self.originalList)

class CutAndPasteWithChildrenIntegrationTest(CommandWithChildrenTestCase):
    def assertTaskListUnchanged(self):
        self.assertTaskList(self.originalList)
        self.failUnlessParentAndChild(self.parent, self.child)
        self.failUnlessParentAndChild(self.child, self.grandchild)

    def testUndoCutAndPaste(self):
        self.cut([self.child])
        self.paste()
        self.undo()
        self.undo()
        self.assertTaskListUnchanged()
        
    def testUndoCutAndPasteAsSubtask(self):
        self.cut([self.child])
        self.paste([self.child2])
        self.undo()
        self.undo()
        self.assertTaskListUnchanged()

    def testUndoCutAndPasteParentAndGrandChild(self):
        self.cut([self.parent, self.grandchild])
        self.paste()
        self.undo()
        self.undo()
        self.assertTaskListUnchanged()

    def testRedoCutAndPasteParentAndGrandChild(self):
        self.cut([self.parent, self.grandchild])
        self.paste()
        self.undo()
        self.undo()
        self.redo()
        self.redo()
        self.assertEqual(5, len(self.taskList))
        self.failIf(self.child.children())
        

class CopyCommandWithTasksTest(TaskCommandTestCase):
    def testCopyTaskWithoutSelection(self):
        self.copy([])
        self.assertDoUndoRedo(
            lambda: self.assertEqual([], command.Clipboard().get()[0]),
                    self.assertTaskList(self.originalList))

    def testCopyTask(self):
        self.copy([self.task1])
        copiedTask = command.Clipboard().get()[0][0]
        self.assertDoUndoRedo(lambda: (self.assertTaskCopy(self.task1, copiedTask),
            self.assertTaskList(self.originalList)),
            lambda: (self.assertTaskList(self.originalList),
            self.failIf(command.Clipboard())))


class CopyCommandWithTasksWithChildrenTest(CommandWithChildrenTestCase):
    def testCopy(self):
        self.copy([self.parent])
        copiedTask = command.Clipboard().get()[0][0]
        self.assertDoUndoRedo(
            lambda: self.assertTaskCopy(self.parent, copiedTask),
            lambda: (self.assertTaskList(self.originalList),
            self.failIf(command.Clipboard())))


class CopyCommandWithEffortTest(CommandWithEffortTestCase):
    def testCopyEffortWithoutSelection(self):
        self.copy([])
        self.assertDoUndoRedo(
            lambda: self.assertEqual([], command.Clipboard().get()[0]),
                    self.assertEffortList(self.originalEffortList))
        
    def testCopyEffort(self):
        self.copy([self.effort1])
        copiedEffort = command.Clipboard().get()[0][0]
        self.assertDoUndoRedo(
            lambda: self.assertEqualEfforts(self.effort1, copiedEffort),
            lambda: (self.assertEffortList(self.originalEffortList),
            self.failIf(command.Clipboard())))
            
    def testCopyMultipleEfforts(self):
        self.copy([self.effort1, self.effort2])
        copiedEfforts = command.Clipboard().get()[0]
        self.assertDoUndoRedo(
            lambda: (self.assertEqualEfforts(self.effort1, copiedEfforts[0]), 
                    self.assertEqualEfforts(self.effort2, copiedEfforts[1])),
            lambda: (self.assertEffortList(self.originalEffortList),
            self.failIf(command.Clipboard())))
     
        
class DragAndDropWithTasksTest(CommandWithChildrenTestCase):
    def dragAndDrop(self, draggedItems, dropItem): # pylint: disable=W0222
        command.DragAndDropTaskCommand(self.taskList, draggedItems, drop=[dropItem]).do()
        
    def testDragAndDropRootTask(self):
        self.taskList.append(self.task2)
        self.dragAndDrop([self.task2], self.task1)
        self.assertDoUndoRedo(
            lambda: self.failUnless(self.task2 in self.task1.children()),
            lambda: self.failIf(self.task2 in self.task1.children()))
            
    def testDontAllowDropOnSelf(self):
        self.dragAndDrop([self.task1], self.task1)
        self.assertDoUndoRedo(lambda: self.failIf(self.task1 in self.task1.children()))
        
    def testDragChildTaskAndDropOnOtherRootTask(self):
        self.dragAndDrop([self.child2], self.task1)
        self.assertDoUndoRedo(
             lambda: (self.failUnless(self.child2 in self.task1.children()),
                     self.failIf(self.child2 in self.parent.children())),
             lambda: (self.failIf(self.child2 in self.task1.children()), 
                     self.failUnless(self.child2 in self.parent.children())))
                     
    def testDragChildAndDropOnOwnParent(self):
        self.dragAndDrop([self.child2], self.parent)
        self.assertDoUndoRedo(
            lambda: self.failUnless(self.child2 in self.parent.children()))
            
    def testDragParentAndDropOnOwnChild(self):
        self.dragAndDrop([self.parent], self.child2) 
        self.assertDoUndoRedo(
            lambda: (self.failUnless(self.child2 in self.parent.children()),
                    self.failIf(self.parent in self.child2.children())))
                    

        



# -*- coding: utf-8 -*-

'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>
Copyright (C) 2008 João Alexandre de Toledo <jtoledo@griffo.com.br>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
import os
from taskcoachlib import meta, patterns, operating_system
from taskcoachlib.i18n import _
from taskcoachlib.domain import date, task
from taskcoachlib.thirdparty.pubsub import pub
import artprovider


class TaskBarIcon(patterns.Observer, wx.TaskBarIcon):
    def __init__(self, mainwindow, taskList, settings,
            defaultBitmap='taskcoach', tickBitmap='clock_icon',
            tackBitmap='clock_stopwatch_icon', *args, **kwargs):
        super(TaskBarIcon, self).__init__(*args, **kwargs)
        self.__window = mainwindow
        self.__taskList = taskList
        self.__settings = settings
        self.__bitmap = self.__defaultBitmap = defaultBitmap
        self.__currentBitmap = self.__bitmap
        self.__tooltipText = ''
        self.__currentText = self.__tooltipText
        self.__tickBitmap = tickBitmap
        self.__tackBitmap = tackBitmap
        self.registerObserver(self.onTaskListChanged,
            eventType=taskList.addItemEventType(), eventSource=taskList)
        self.registerObserver(self.onTaskListChanged,
            eventType=taskList.removeItemEventType(), eventSource=taskList)
        pub.subscribe(self.onTrackingChanged,
                      task.Task.trackingChangedEventType())
        pub.subscribe(self.onChangeDueDateTime,
                      task.Task.dueDateTimeChangedEventType())
        # When the user chances the due soon hours preferences it may cause
        # a task to change appearance. That also means the number of due soon
        # tasks has changed, so we need to change the tool tip text.
        # Note that directly subscribing to the setting (behavior.duesoonhours)
        # is not reliable. The TaskBarIcon may get the event before the tasks
        # do. When that happens the tasks haven't changed their status yet and
        # we would use the wrong status count.
        self.registerObserver(self.onChangeDueDateTime_Deprecated,
            eventType=task.Task.appearanceChangedEventType())
        if operating_system.isGTK():
            events = [wx.EVT_TASKBAR_LEFT_DOWN]
        elif operating_system.isWindows():
            # See http://msdn.microsoft.com/en-us/library/windows/desktop/aa511448.aspx#interaction
            events = [wx.EVT_TASKBAR_LEFT_DOWN, wx.EVT_TASKBAR_LEFT_DCLICK]
        else:
            events = [wx.EVT_TASKBAR_LEFT_DCLICK]
        for event in events:
            self.Bind(event, self.onTaskbarClick)
        self.__setTooltipText()
        mainwindow.Bind(wx.EVT_IDLE, self.onIdle)

    # Event handlers:

    def onIdle(self, event):
        if self.__currentText != self.__tooltipText or self.__currentBitmap != self.__bitmap:
            self.__currentText = self.__tooltipText
            self.__currentBitmap = self.__bitmap
            self.__setIcon()
        if event is not None: # Unit tests
            event.Skip()

    def onTaskListChanged(self, event):  # pylint: disable=W0613
        self.__setTooltipText()
        self.__startOrStopTicking()

    def onTrackingChanged(self, newValue, sender):
        if newValue:
            self.registerObserver(self.onChangeSubject,
                                  eventType=sender.subjectChangedEventType(),
                                  eventSource=sender)
        else:
            self.removeObserver(self.onChangeSubject,
                                eventType=sender.subjectChangedEventType())
        self.__setTooltipText()
        if newValue:
            self.__startTicking()
        else:
            self.__stopTicking()

    def onChangeSubject(self, event):  # pylint: disable=W0613
        self.__setTooltipText()

    def onChangeDueDateTime(self, newValue, sender):  # pylint: disable=W0613
        self.__setTooltipText()

    def onChangeDueDateTime_Deprecated(self, event):
        self.__setTooltipText()

    def onEverySecond(self):
        if self.__settings.getboolean('window',
            'blinktaskbariconwhentrackingeffort') and \
            not operating_system.isMacOsXMavericks_OrNewer():
            self.__toggleTrackingBitmap()
            self.__setIcon()

    def onTaskbarClick(self, event):
        if self.__window.IsIconized() or not self.__window.IsShown():
            self.__window.restore(event)
        else:
            if operating_system.isMac():
                self.__window.Raise()
            else:
                self.__window.Iconize()

    # Menu:

    def setPopupMenu(self, menu):
        self.Bind(wx.EVT_TASKBAR_RIGHT_UP, self.popupTaskBarMenu)
        self.popupmenu = menu  # pylint: disable=W0201

    def popupTaskBarMenu(self, event):  # pylint: disable=W0613
        self.PopupMenu(self.popupmenu)

    # Getters:

    def tooltip(self):
        return self.__tooltipText

    def bitmap(self):
        return self.__bitmap

    def defaultBitmap(self):
        return self.__defaultBitmap

    # Private methods:

    def __startOrStopTicking(self):
        self.__startTicking()
        self.__stopTicking()

    def __startTicking(self):
        if self.__taskList.nrBeingTracked() > 0:
            self.startClock()
            self.__toggleTrackingBitmap()
            self.__setIcon()

    def startClock(self):
        date.Scheduler().schedule_interval(self.onEverySecond, seconds=1)

    def __stopTicking(self):
        if self.__taskList.nrBeingTracked() == 0:
            self.stopClock()
            self.__setDefaultBitmap()
            self.__setIcon()

    def stopClock(self):
        date.Scheduler().unschedule(self.onEverySecond)

    toolTipMessages = \
        [(task.status.overdue, _('one task overdue'), _('%d tasks overdue')),
         (task.status.duesoon, _('one task due soon'), _('%d tasks due soon'))]

    def __setTooltipText(self):
        ''' Note that Windows XP and Vista limit the text shown in the
            tool tip to 64 characters, so we cannot show everything we would
            like to and have to make choices. '''
        textParts = []
        trackedTasks = self.__taskList.tasksBeingTracked()
        if trackedTasks:
            count = len(trackedTasks)
            if count == 1:
                tracking = _('tracking "%s"') % trackedTasks[0].subject()
            else:
                tracking = _('tracking effort for %d tasks') % count
            textParts.append(tracking)
        else:
            counts = self.__taskList.nrOfTasksPerStatus()
            for status, singular, plural in self.toolTipMessages:
                count = counts[status]
                if count == 1:
                    textParts.append(singular)
                elif count > 1:
                    textParts.append(plural % count)

        textPart = ', '.join(textParts)
        filename = os.path.basename(self.__window.taskFile.filename())
        namePart = u'%s - %s' % (meta.name, filename) if filename else meta.name
        text = u'%s\n%s' % (namePart, textPart) if textPart else namePart

        if text != self.__tooltipText:
            self.__tooltipText = text

    def __setDefaultBitmap(self):
        self.__bitmap = self.__defaultBitmap

    def __toggleTrackingBitmap(self):
        tick, tack = self.__tickBitmap, self.__tackBitmap
        self.__bitmap = tack if self.__bitmap == tick else tick

    def __setIcon(self):
        icon = artprovider.getIcon(self.__bitmap)
        self.SetIcon(icon, self.__tooltipText)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from wxSchedulerPrint import *
import calendar
import wx


class wxReportScheduler( wx.Printout ):
	"""
	This is a class which demonstrate how to use the in-memory wxSchedulerPrint() 
	object on wxPython printing framework.
	You can control wxScheduler in the same way on GUI.
	For other info on wxPrintOut class and methods check the wxPython 
	documentation (RTFM for nerds ;-) ).
	"""
	def __init__( self, format, style, drawerClass, day, weekstart, periodCount, schedules ):
		self._format	= format
		self._style = style
		self._drawerClass = drawerClass
		self._day		= day
		self._schedules	= schedules
		self._weekstart = weekstart
		self._periodCount = periodCount
		self.pages		= None
		self._bitmaps = []

		wx.Printout.__init__( self )

	def _DrawPages(self, dc):
		scheduler = self._GetScheduler( dc, self._day )
		scheduler.Draw( None )
		self.pages = scheduler.pageCount

		for idx in xrange( scheduler.pageCount ):
			self._bitmaps.append( scheduler.Draw( idx + 1 ) )

	def _GetScheduler( self, dc, day ):
		"""
		Return an in-memory wxSchedulerPrint() object for adding 
		schedules and print on current wxDC
		"""

		scheduler = wxSchedulerPrint( dc )
		scheduler.SetViewType( self._format )
		scheduler.SetStyle( self._style )
		scheduler.SetDrawer( self._drawerClass )
		scheduler.SetDate( day )
		scheduler.SetWeekStart( self._weekstart )
		scheduler.SetPeriodCount( self._periodCount )

		for schedule in self._schedules:
			scheduler.Add( schedule )

		return scheduler

	def OnPrintPage( self, page ):
		"""
		This code draw a wxScheduler scaled to fit page using date, format and 
		schedules passed by the user.
		Note there is no difference on manage scheduler and schedules beetwen 
		GUI and printing framework
		"""

		dc = self.GetDC()
		if self.pages is None:
			self._DrawPages( dc )

		dc.DrawBitmap(self._bitmaps[page - 1], 0, 0, False)

		return True

	def HasPage( self, page ):
		if self.pages is None:
			self._DrawPages( self.GetDC() )
		return page <= self.pages

	def GetPageInfo( self ):
		if self.pages is None:
			self._DrawPages( self.GetDC() )

		return ( 1, self.pages, 1, 1 )



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib import meta, patterns, operating_system
from taskcoachlib.i18n import _
from taskcoachlib.thirdparty.pubsub import pub
from taskcoachlib.workarounds import ExceptionAsUnicode
import ConfigParser
import os
import sys
import wx
import shutil
import defaults


class UnicodeAwareConfigParser(ConfigParser.RawConfigParser):
    def set(self, section, setting, value):  # pylint: disable=W0222
        if type(value) == type(u''):
            value = value.encode('utf-8')
        ConfigParser.RawConfigParser.set(self, section, setting, value)

    def get(self, section, setting):  # pylint: disable=W0221
        value = ConfigParser.RawConfigParser.get(self, section, setting)
        return value.decode('utf-8')  # pylint: disable=E1103


class CachingConfigParser(UnicodeAwareConfigParser):
    ''' ConfigParser is rather slow, so cache its values. '''
    def __init__(self, *args, **kwargs):
        self.__cachedValues = dict()
        UnicodeAwareConfigParser.__init__(self, *args, **kwargs)
        
    def read(self, *args, **kwargs):
        self.__cachedValues = dict()
        return UnicodeAwareConfigParser.read(self, *args, **kwargs)

    def set(self, section, setting, value):
        self.__cachedValues[(section, setting)] = value
        UnicodeAwareConfigParser.set(self, section, setting, value)
        
    def get(self, section, setting):
        cache, key = self.__cachedValues, (section, setting)
        if key not in cache:
            cache[key] = UnicodeAwareConfigParser.get(self, *key)  # pylint: disable=W0142
        return cache[key]
        
        
class Settings(object, CachingConfigParser):
    def __init__(self, load=True, iniFile=None, *args, **kwargs):
        # Sigh, ConfigParser.SafeConfigParser is an old-style class, so we 
        # have to call the superclass __init__ explicitly:
        CachingConfigParser.__init__(self, *args, **kwargs)
        self.initializeWithDefaults()
        self.__loadAndSave = load
        self.__iniFileSpecifiedOnCommandLine = iniFile
        self.migrateConfigurationFiles()
        if load:
            # First, try to load the settings file from the program directory,
            # if that fails, load the settings file from the settings directory
            try:
                if not self.read(self.filename(forceProgramDir=True)):
                    self.read(self.filename())
                errorMessage = ''
            except ConfigParser.ParsingError, errorMessage:
                # Ignore exceptions and simply use default values. 
                # Also record the failure in the settings:
                self.initializeWithDefaults()
            self.setLoadStatus(ExceptionAsUnicode(errorMessage))
        else:
            # Assume that if the settings are not to be loaded, we also 
            # should be quiet (i.e. we are probably in test mode):
            self.__beQuiet()
        pub.subscribe(self.onSettingsFileLocationChanged, 
                      'settings.file.saveinifileinprogramdir')
        
    def onSettingsFileLocationChanged(self, value):
        saveIniFileInProgramDir = value
        if not saveIniFileInProgramDir:
            try:
                os.remove(self.generatedIniFilename(forceProgramDir=True))
            except: 
                return  # pylint: disable=W0702
            
    def initializeWithDefaults(self):
        for section in self.sections():
            self.remove_section(section)
        for section, settings in defaults.defaults.items():
            self.add_section(section)
            for key, value in settings.items():
                # Don't notify observers while we are initializing
                super(Settings, self).set(section, key, value)
                
    def setLoadStatus(self, message):
        self.set('file', 'inifileloaded', 'False' if message else 'True')
        self.set('file', 'inifileloaderror', message)

    def __beQuiet(self):
        noisySettings = [('window', 'splash', 'False'), 
                         ('window', 'tips', 'False'), 
                         ('window', 'starticonized', 'Always')]
        for section, setting, value in noisySettings:
            self.set(section, setting, value)
            
    def add_section(self, section, copyFromSection=None):  # pylint: disable=W0221
        result = super(Settings, self).add_section(section)
        if copyFromSection:
            for name, value in self.items(copyFromSection):
                super(Settings, self).set(section, name, value)
        return result
    
    def getRawValue(self, section, option):
        return super(Settings, self).get(section, option)
    
    def init(self, section, option, value):
        return super(Settings, self).set(section, option, value)

    def get(self, section, option):
        try:
            result = super(Settings, self).get(section, option)
        except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
            return self.getDefault(section, option)
        result = self._fixValuesFromOldIniFiles(section, option, result)
        result = self._ensureMinimum(section, option, result)
        return result

    def getDefault(self, section, option):
        defaultSectionKey = section.strip('0123456789')
        try:
            defaultSection = defaults.defaults[defaultSectionKey]
        except KeyError:
            raise ConfigParser.NoSectionError, defaultSectionKey
        try:
            return defaultSection[option]
        except KeyError:
            raise ConfigParser.NoOptionError, (option, defaultSection)
            
    def _ensureMinimum(self, section, option, result):
        # Some settings may have a minimum value, make sure we return at 
        # least that minimum value:
        if section in defaults.minimum and option in defaults.minimum[section]:
            result = max(result, defaults.minimum[section][option])
        return result
    
    def _fixValuesFromOldIniFiles(self, section, option, result):
        ''' Try to fix settings from old TaskCoach.ini files that are no longer 
            valid. '''
        original = result
        # Starting with release 1.1.0, the date properties of tasks (startDate,
        # dueDate and completionDate) are datetimes: 
        taskDateColumns = ('startDate', 'dueDate', 'completionDate')
        orderingViewers = ['taskviewer', 'categoryviewer', 'noteviewer',
                           'noteviewerintaskeditor', 'noteviewerincategoryeditor',
                           'noteviewerinattachmenteditor', 'categoryviewerintaskeditor',
                           'categoryviewerinnoteeditor']
        if option == 'sortby':
            if result in taskDateColumns:
                result += 'Time'
            try:
                eval(result)
            except:
                sortKeys = [result]
                try:
                    ascending = self.getboolean(section, 'sortascending')
                except:
                    ascending = True
                result = '["%s%s"]' % (('' if ascending else '-'), result)
        elif option == 'columns':
            columns = [(col + 'Time' if col in taskDateColumns else col) for col in eval(result)]
            result = str(columns)
        elif option == 'columnwidths':
            widths = dict()
            try:
                columnWidthMap = eval(result)
            except SyntaxError:
                columnWidthMap = dict()
            for column, width in columnWidthMap.items():
                if column in taskDateColumns:
                    column += 'Time'
                widths[column] = width
            if section in orderingViewers and 'ordering' not in widths:
                widths['ordering'] = 28
            result = str(widths)
        elif section == 'feature' and option == 'notifier' and result == 'Native':
            result = 'Task Coach'
        elif section == 'editor' and option == 'preferencespages':
            result = result.replace('colors', 'appearance')
        elif section in orderingViewers and option == 'columnsalwaysvisible':
            # XXX: remove 'ordering' from always visible columns. This wasn't in any official release
            # but I need it so that people can test without resetting their .ini file...
            # Remove this after the 1.3.38 release.
            try:
                columns = eval(result)
            except SyntaxError:
                columns = ['ordering']
            else:
                if 'ordering' in columns:
                    columns.remove('ordering')
            result = str(columns)
        if result != original:
            super(Settings, self).set(section, option, result)
        return result

    def set(self, section, option, value, new=False):  # pylint: disable=W0221
        if new:
            currentValue = 'a new option, so use something as current value'\
                ' that is unlikely to be equal to the new value'
        else:
            currentValue = self.get(section, option)
        if value != currentValue:
            super(Settings, self).set(section, option, value)
            patterns.Event('%s.%s' % (section, option), self, value).send()
            return True
        else:
            return False
            
    def setboolean(self, section, option, value):
        if self.set(section, option, str(value)):
            pub.sendMessage('settings.%s.%s' % (section, option), value=value)

    setvalue = settuple = setlist = setdict = setint = setboolean
            
    def settext(self, section, option, value):
        if self.set(section, option, value):
            pub.sendMessage('settings.%s.%s' % (section, option), value=value)
                
    def getlist(self, section, option):
        return self.getEvaluatedValue(section, option, eval)
        
    getvalue = gettuple = getdict = getlist

    def getint(self, section, option):
        return self.getEvaluatedValue(section, option, int)
    
    def getboolean(self, section, option):
        return self.getEvaluatedValue(section, option, self.evalBoolean)
    
    def gettext(self, section, option):
        return self.get(section, option)

    @staticmethod
    def evalBoolean(stringValue):
        if stringValue in ('True', 'False'):
            return 'True' == stringValue
        else:
            raise ValueError("invalid literal for Boolean value: '%s'" % stringValue)
         
    def getEvaluatedValue(self, section, option, evaluate=eval, showerror=wx.MessageBox):
        stringValue = self.get(section, option)
        try:
            return evaluate(stringValue)
        except Exception, exceptionMessage:  # pylint: disable=W0703
            message = '\n'.join([ \
                _('Error while reading the %s-%s setting from %s.ini.') % (section, option, meta.filename),
                _('The value is: %s') % stringValue,
                _('The error is: %s') % exceptionMessage,
                _('%s will use the default value for the setting and should proceed normally.') % meta.name])
            showerror(message, caption=_('Settings error'), style=wx.ICON_ERROR)
            defaultValue = self.getDefault(section, option)
            self.set(section, option, defaultValue, new=True)  # Ignore current value
            return evaluate(defaultValue)
        
    def save(self, showerror=wx.MessageBox, file=file):  # pylint: disable=W0622
        self.set('version', 'python', sys.version)
        self.set('version', 'wxpython', '%s-%s @ %s' % (wx.VERSION_STRING, wx.PlatformInfo[2], wx.PlatformInfo[1]))
        self.set('version', 'pythonfrozen', str(hasattr(sys, 'frozen')))
        self.set('version', 'current', meta.data.version)
        if not self.__loadAndSave:
            return
        try:
            path = self.path()
            if not os.path.exists(path):
                os.mkdir(path)
            tmpFile = file(self.filename() + '.tmp', 'w')
            self.write(tmpFile)
            tmpFile.close()
            if os.path.exists(self.filename()):
                os.remove(self.filename())
            os.rename(self.filename() + '.tmp', self.filename())
        except Exception, message:  # pylint: disable=W0703
            showerror(_('Error while saving %s.ini:\n%s\n') % \
                      (meta.filename, message), caption=_('Save error'), 
                      style=wx.ICON_ERROR)

    def filename(self, forceProgramDir=False):
        if self.__iniFileSpecifiedOnCommandLine:
            return self.__iniFileSpecifiedOnCommandLine
        else:
            return self.generatedIniFilename(forceProgramDir) 
    
    def path(self, forceProgramDir=False, environ=os.environ):  # pylint: disable=W0102
        if self.__iniFileSpecifiedOnCommandLine:
            return self.pathToIniFileSpecifiedOnCommandLine()
        elif forceProgramDir or self.getboolean('file', 
                                                'saveinifileinprogramdir'):
            return self.pathToProgramDir()
        else:
            return self.pathToConfigDir(environ)

    @staticmethod
    def pathToDocumentsDir():
        if operating_system.isWindows():
            from win32com.shell import shell, shellcon
            try:
                return shell.SHGetSpecialFolderPath(None, shellcon.CSIDL_PERSONAL)
            except:
                # Yes, one of the documented ways to get this sometimes fail with "Unspecified error". Not sure
                # this will work either.
                # Update: There are cases when it doesn't work either; see support request #410...
                try:
                    return shell.SHGetFolderPath(None, shellcon.CSIDL_PERSONAL, None, 0) # SHGFP_TYPE_CURRENT not in shellcon
                except:
                    return os.getcwd() # Fuck this
        elif operating_system.isMac():
            import Carbon.Folder, Carbon.Folders, Carbon.File
            pathRef = Carbon.Folder.FSFindFolder(Carbon.Folders.kUserDomain, Carbon.Folders.kDocumentsFolderType, True)
            return Carbon.File.pathname(pathRef)
        elif operating_system.isGTK():
            try:
                from PyKDE4.kdeui import KGlobalSettings
            except ImportError:
                pass
            else:
                return unicode(KGlobalSettings.documentPath())
        # Assuming Unix-like
        return os.path.expanduser('~')

    def pathToProgramDir(self):
        path = sys.argv[0]
        if not os.path.isdir(path):
            path = os.path.dirname(path)
        return path

    def pathToConfigDir(self, environ):
        try:
            if operating_system.isGTK():
                from taskcoachlib.thirdparty.xdg import BaseDirectory
                path = BaseDirectory.save_config_path(meta.name)
            elif operating_system.isMac():
                import Carbon.Folder, Carbon.Folders, Carbon.File
                pathRef = Carbon.Folder.FSFindFolder(Carbon.Folders.kUserDomain, Carbon.Folders.kPreferencesFolderType, True)
                path = Carbon.File.pathname(pathRef)
                # XXXFIXME: should we release pathRef ? Doesn't seem so since I get a SIGSEGV if I try.
            elif operating_system.isWindows():
                from win32com.shell import shell, shellcon
                path = os.path.join(shell.SHGetSpecialFolderPath(None, shellcon.CSIDL_APPDATA, True), meta.name)
            else:
                path = self.pathToConfigDir_deprecated(environ=environ)
        except: # Fallback to old dir
            path = self.pathToConfigDir_deprecated(environ=environ)
        return path

    def _pathToDataDir(self, *args, **kwargs):
        forceGlobal = kwargs.pop('forceGlobal', False)
        if operating_system.isGTK():
            from taskcoachlib.thirdparty.xdg import BaseDirectory
            path = BaseDirectory.save_data_path(meta.name)
        elif operating_system.isMac():
            import Carbon.Folder, Carbon.Folders, Carbon.File
            pathRef = Carbon.Folder.FSFindFolder(Carbon.Folders.kUserDomain, Carbon.Folders.kApplicationSupportFolderType, True)
            path = Carbon.File.pathname(pathRef)
            # XXXFIXME: should we release pathRef ? Doesn't seem so since I get a SIGSEGV if I try.
            path = os.path.join(path, meta.name)
        elif operating_system.isWindows():
            if self.__iniFileSpecifiedOnCommandLine and not forceGlobal:
                path = self.pathToIniFileSpecifiedOnCommandLine()
            else:
                from win32com.shell import shell, shellcon
                path = os.path.join(shell.SHGetSpecialFolderPath(None, shellcon.CSIDL_APPDATA, True), meta.name)

        else: # Errr...
            path = self.path()

        if operating_system.isWindows():
            # Follow shortcuts.
            from win32com.client import Dispatch
            shell = Dispatch('WScript.Shell')
            for component in args:
                path = os.path.join(path, component)
                if os.path.exists(path + '.lnk'):
                    shortcut = shell.CreateShortcut(path + '.lnk')
                    path = shortcut.TargetPath
        else:
            path = os.path.join(path, *args)

        exists = os.path.exists(path)
        if not exists:
            os.makedirs(path)
        return path, exists

    def pathToDataDir(self, *args, **kwargs):
        return self._pathToDataDir(*args, **kwargs)[0]

    def _pathToTemplatesDir(self):
        try:
            return self._pathToDataDir('templates')
        except:
            pass # Fallback on old path
        return self.pathToTemplatesDir_deprecated(), True

    def pathToTemplatesDir(self):
        return self._pathToTemplatesDir()[0]

    def pathToBackupsDir(self):
        return self._pathToDataDir('backups')[0]

    def pathToConfigDir_deprecated(self, environ):
        try:
            path = os.path.join(environ['APPDATA'], meta.filename)
        except Exception:
            path = os.path.expanduser("~")  # pylint: disable=W0702
            if path == "~":
                # path not expanded: apparently, there is no home dir
                path = os.getcwd()
            path = os.path.join(path, '.%s' % meta.filename)
        return operating_system.decodeSystemString(path)

    def pathToTemplatesDir_deprecated(self, doCreate=True):
        path = os.path.join(self.path(), 'taskcoach-templates')

        if operating_system.isWindows():
            # Under Windows, check for a shortcut and follow it if it
            # exists.

            if os.path.exists(path + '.lnk'):
                from win32com.client import Dispatch  # pylint: disable=F0401

                shell = Dispatch('WScript.Shell')
                shortcut = shell.CreateShortcut(path + '.lnk')
                return shortcut.TargetPath

        if doCreate:
            try:
                os.makedirs(path)
            except OSError:
                pass
        return operating_system.decodeSystemString(path)

    def pathToIniFileSpecifiedOnCommandLine(self):
        return os.path.dirname(self.__iniFileSpecifiedOnCommandLine) or '.'
    
    def generatedIniFilename(self, forceProgramDir):
        return os.path.join(self.path(forceProgramDir), '%s.ini' % meta.filename)

    def migrateConfigurationFiles(self):
        # Templates. Extra care for Windows shortcut.
        oldPath = self.pathToTemplatesDir_deprecated(doCreate=False)
        newPath, exists = self._pathToTemplatesDir()
        if self.__iniFileSpecifiedOnCommandLine:
            globalPath = os.path.join(self.pathToDataDir(forceGlobal=True), 'templates')
            if os.path.exists(globalPath) and not os.path.exists(oldPath):
                # Upgrade from fresh installation of 1.3.24 Portable
                oldPath = globalPath
                if exists and not os.path.exists(newPath + '-old'):
                    # WTF?
                    os.rename(newPath, newPath + '-old')
                exists = False
        if exists:
            return
        if oldPath != newPath:
            if operating_system.isWindows() and os.path.exists(oldPath + '.lnk'):
                shutil.move(oldPath + '.lnk', newPath + '.lnk')
            elif os.path.exists(oldPath):
                # pathToTemplatesDir() has created the directory
                try:
                    os.rmdir(newPath)
                except:
                    pass
                shutil.move(oldPath, newPath)
        # Ini file
        oldPath = os.path.join(self.pathToConfigDir_deprecated(environ=os.environ), '%s.ini' % meta.filename)
        newPath = os.path.join(self.pathToConfigDir(environ=os.environ), '%s.ini' % meta.filename)
        if newPath != oldPath and os.path.exists(oldPath):
            shutil.move(oldPath, newPath)
        # Cleanup
        try:
            os.rmdir(self.pathToConfigDir_deprecated(environ=os.environ))
        except:
            pass



#!/usr/bin/env python

'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import os
import sys

# Workaround for a bug in Ubuntu 10.10
os.environ['XLIB_SKIP_ARGB_VISUALS'] = '1'

# This prevents a message printed to the console when wx.lib.masked
# is imported from taskcoachlib.widgets on Ubuntu 12.04 64 bits...
try:
    from mx import DateTime
except ImportError:
    pass


if not hasattr(sys, "frozen"):
    # These checks are only necessary in a non-frozen environment, i.e. we
    # skip these checks when run from a py2exe-fied application
    import wxversion
    wxversion.select(["2.8-unicode", "3.0"], optionsRequired=True)
    try:
        import taskcoachlib  # pylint: disable=W0611
    except ImportError:
        # On Ubuntu 12.04, taskcoachlib is installed in /usr/share/pyshared,
        # but that folder is not on the python path. Don't understand why. 
        # We'll add it manually so the application can find it.
        sys.path.insert(0, '/usr/share/pyshared')
        try:
            import taskcoachlib  # pylint: disable=W0611
        except ImportError:
            sys.stderr.write('''ERROR: cannot import the library 'taskcoachlib'.
Please see https://answers.launchpad.net/taskcoach/+faq/1063 
for more information and possible resolutions.
''')
            sys.exit(1)


def start():
    ''' Process command line options and start the application. '''

    # pylint: disable=W0404
    from taskcoachlib import config, application 
    options, args = config.ApplicationOptionParser().parse_args()
    app = application.Application(options, args)
    if options.profile:
        import cProfile
        cProfile.runctx('app.start()', globals(), locals(), filename='.profile')
    else:
        app.start()


if __name__ == '__main__':
    start()



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib import patterns
import weakref
from taskcoachlib.thirdparty._weakrefset import WeakSet


class Attribute(object):
    __slots__ = ('__value', '__owner', '__setEvent')
    
    def __init__(self, value, owner, setEvent):
        super(Attribute, self).__init__()
        self.__value = value
        self.__owner = weakref.ref(owner)
        self.__setEvent = setEvent.im_func
        
    def get(self):
        return self.__value
    
    @patterns.eventSource
    def set(self, value, event=None):
        owner = self.__owner()
        if owner is not None:
            if value == self.__value:
                return False
            self.__value = value
            self.__setEvent(owner, event)
            return True
    

class SetAttribute(object):
    __slots__ = ('__set', '__owner', '__addEvent', '__removeEvent', 
                 '__changeEvent', '__setClass')
    
    def __init__(self, values, owner, addEvent=None, removeEvent=None, changeEvent=None, weak=False):
        self.__setClass = WeakSet if weak else set
        self.__set = self.__setClass(values) if values else self.__setClass()
        self.__owner = weakref.ref(owner)
        self.__addEvent = (addEvent or self.__nullEvent).im_func
        self.__removeEvent = (removeEvent or self.__nullEvent).im_func
        self.__changeEvent = (changeEvent or self.__nullEvent).im_func
        
    def get(self):
        return set(self.__set)
    
    @patterns.eventSource
    def set(self, values, event=None):
        owner = self.__owner()
        if owner is not None:
            if values == set(self.__set):
                return False
            added = values - set(self.__set)
            removed = set(self.__set) - values
            self.__set = self.__setClass(values)
            if added:
                self.__addEvent(owner, event, *added) # pylint: disable=W0142
            if removed:
                self.__removeEvent(owner, event, *removed) # pylint: disable=W0142
            if added or removed:
                self.__changeEvent(owner, event, *set(self.__set))
            return True
    
    @patterns.eventSource            
    def add(self, values, event=None):
        owner = self.__owner()
        if owner is not None:
            if values <= set(self.__set):
                return False
            self.__set = self.__setClass(set(self.__set) | values)
            self.__addEvent(owner, event, *values) # pylint: disable=W0142
            self.__changeEvent(owner, event, *set(self.__set))
            return True
    
    @patterns.eventSource                    
    def remove(self, values, event=None):
        owner = self.__owner()
        if owner is not None:
            if values & set(self.__set) == set():
                return False
            self.__set = self.__setClass(set(self.__set) - values)
            self.__removeEvent(owner, event, *values) # pylint: disable=W0142
            self.__changeEvent(owner, event, *set(self.__set))
            return True

    def __nullEvent(self, *args, **kwargs):
        pass



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>
Copyright (C) 2008 Rob McMullen <rob.mcmullen@gmail.com>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import base_uicommand
import wx


class SettingsCommand(base_uicommand.UICommand):  # pylint: disable=W0223
    ''' SettingsCommands are saved in the settings (a ConfigParser). '''

    def __init__(self, settings=None, setting=None, section='view', 
                 *args, **kwargs):
        self.settings = settings
        self.section = section
        self.setting = setting
        super(SettingsCommand, self).__init__(*args, **kwargs)


class BooleanSettingsCommand(SettingsCommand):  # pylint: disable=W0223
    ''' Bae class for commands that change a boolean setting. 
        Whenever the setting is changed, the user interface 
        representation is changed as well. E.g. a menu gets 
        a checkmark. '''

    def __init__(self, value=None, *args, **kwargs):
        self.value = value
        super(BooleanSettingsCommand, self).__init__(*args, **kwargs)
        
    def onUpdateUI(self, event):
        event.Check(self.isSettingChecked())
        super(BooleanSettingsCommand, self).onUpdateUI(event)

    def addToMenu(self, menu, window, position=None):
        menuId = super(BooleanSettingsCommand, self).addToMenu(menu, window, 
                                                              position)
        menuItem = menu.FindItemById(menuId)
        menuItem.Check(self.isSettingChecked())
        
    def isSettingChecked(self):
        raise NotImplementedError  # pragma: no cover
    

class UICheckCommand(BooleanSettingsCommand):
    def __init__(self, *args, **kwargs):
        kwargs['bitmap'] = kwargs.get('bitmap', self.getBitmap())
        super(UICheckCommand, self).__init__(kind=wx.ITEM_CHECK, 
            *args, **kwargs)
        
    def isSettingChecked(self):
        return self.settings.getboolean(self.section, self.setting)

    def _isMenuItemChecked(self, event):
        # There's a bug in wxPython 2.8.3 on Windows XP that causes 
        # event.IsChecked() to return the wrong value in the context menu.
        # The menu on the main window works fine. So we first try to access the
        # context menu to get the checked state from the menu item itself.
        # This will fail if the event is coming from the window, but in that
        # case we can event.IsChecked() expect to work so we use that.
        try:
            return event.GetEventObject().FindItemById(event.GetId()).IsChecked()
        except AttributeError:
            return event.IsChecked()
        
    def doCommand(self, event):
        self.settings.setboolean(self.section, self.setting, 
            self._isMenuItemChecked(event))
        
    def getBitmap(self):
        # Using our own bitmap for checkable menu items does not work on
        # all platforms, most notably Gtk where providing our own bitmap causes
        # "(python:8569): Gtk-CRITICAL **: gtk_check_menu_item_set_active: 
        # assertion `GTK_IS_CHECK_MENU_ITEM (check_menu_item)' failed"
        return 'nobitmap'


class UIRadioCommand(BooleanSettingsCommand):
    def __init__(self, *args, **kwargs):
        super(UIRadioCommand, self).__init__(kind=wx.ITEM_RADIO, bitmap='', 
                                             *args, **kwargs)
        
    def onUpdateUI(self, event):
        if self.isSettingChecked():
            super(UIRadioCommand, self).onUpdateUI(event)

    def isSettingChecked(self):
        return self.settings.get(self.section, self.setting) == str(self.value)

    def doCommand(self, event):
        self.settings.setvalue(self.section, self.setting, self.value)



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from unittests import asserts
from CommandTestCase import CommandTestCase
from taskcoachlib import command, patterns, config
from taskcoachlib.domain import task, effort, date, category, attachment


class TaskCommandTestCase(CommandTestCase, asserts.Mixin):
    def setUp(self):
        super(TaskCommandTestCase, self).setUp()
        self.settings = task.Task.settings = config.Settings(load=False)
        self.list = self.taskList = task.TaskList()
        self.categories = category.CategoryList()
        self.category = category.Category('cat')
        self.categories.append(self.category)
        self.task1 = task.Task('task1', plannedStartDateTime=date.Now(), 
                               dueDateTime=date.Now() + date.ONE_HOUR)
        self.task2 = task.Task('task2', plannedStartDateTime=date.Now())
        self.taskList.append(self.task1)
        self.originalList = [self.task1]
        
    def tearDown(self):
        super(TaskCommandTestCase, self).tearDown()
        command.Clipboard().clear()

    def delete(self, items=None, shadow=False):
        if items == 'all':
            items = list(self.list)
        command.DeleteTaskCommand(self.list, items or [], shadow=shadow).do()
 
    def paste(self, items=None):  # pylint: disable=W0221
        if items:
            command.PasteAsSubItemCommand(self.taskList, items).do()
        else:
            super(TaskCommandTestCase, self).paste()

    def copy(self, items=None):
        command.CopyCommand(self.list, items or []).do()
        
    def markCompleted(self, tasks=None):
        command.MarkCompletedCommand(self.taskList, tasks or []).do()
        
    def editPercentageComplete(self, tasks=None, percentage=50):
        command.EditPercentageCompleteCommand(self.taskList, tasks or [], 
                                              newValue=percentage).do()
                                              
    def markActive(self, tasks=None):
        command.MarkActiveCommand(self.taskList, tasks or []).do()
        
    def markInactive(self, tasks=None):
        command.MarkInactiveCommand(self.taskList, tasks or []).do()

    def newSubTask(self, tasks=None, markCompleted=False):
        tasks = tasks or []
        newSubTask = command.NewSubTaskCommand(self.taskList, tasks)
        if markCompleted:
            for subtask in newSubTask.items:
                subtask.setCompletionDateTime()
        newSubTask.do()

    def dragAndDrop(self, dropTarget, tasks=None):
        command.DragAndDropTaskCommand(self.taskList, tasks or [], 
                                       drop=dropTarget).do()
        
    def editPlannedStart(self, newPlannedStartDateTime, tasks=None, keep_delta=False):
        command.EditPlannedStartDateTimeCommand(self.taskList, tasks or [],
                                                newValue=newPlannedStartDateTime,
                                                keep_delta=keep_delta).do()

    def editDue(self, newDueDateTime, tasks=None, keep_delta=False):
        command.EditDueDateTimeCommand(self.taskList, tasks or [],
                                       newValue=newDueDateTime, keep_delta=keep_delta).do()


class CommandWithChildrenTestCase(TaskCommandTestCase):
    def setUp(self):
        super(CommandWithChildrenTestCase, self).setUp()
        self.parent = task.Task('parent')
        self.child = task.Task('child')
        self.parent.addChild(self.child)
        self.child2 = task.Task('child2')
        self.parent.addChild(self.child2)
        self.grandchild = task.Task('grandchild')
        self.child.addChild(self.grandchild)
        self.originalList.extend([self.parent, self.child, self.child2, self.grandchild])
        self.taskList.append(self.parent)
        

class CommandWithEffortTestCase(TaskCommandTestCase):
    def setUp(self):
        super(CommandWithEffortTestCase, self).setUp()
        self.list = self.effortList = effort.EffortList(self.taskList)
        self.effort1 = effort.Effort(self.task1)
        self.task1.addEffort(self.effort1)
        self.effort2 = effort.Effort(self.task2, 
            date.DateTime(2004, 1, 1), date.DateTime(2004, 1, 2))
        self.task2.addEffort(self.effort2)
        self.taskList.append(self.task2)
        self.originalEffortList = [self.effort1, self.effort2]


class DeleteCommandWithTasksTest(TaskCommandTestCase):
    def testDeleteAllTasks(self):
        self.taskList.append(self.task2)
        self.delete('all')
        self.assertDoUndoRedo(self.assertEmptyTaskList, 
            lambda: self.assertTaskList([self.task1, self.task2]))

    def testDeleteOneTask(self):
        self.taskList.append(self.task2)
        self.delete([self.task1])
        self.assertDoUndoRedo(lambda: self.assertTaskList([self.task2]),
            lambda: self.assertTaskList([self.task1, self.task2]))

    def testDeleteEmptyList(self):
        self.taskList.remove(self.task1)
        self.delete('all')
        self.assertDoUndoRedo(self.assertEmptyTaskList)

    def testDeleteEmptyList_NoCommandHistory(self):
        self.taskList.remove(self.task1)
        self.delete('all')
        self.assertDoUndoRedo(lambda: self.assertHistoryAndFuture([], []))

    def testDelete(self):
        self.delete('all')
        self.assertDoUndoRedo(self.assertEmptyTaskList,
            lambda: self.assertTaskList(self.originalList))
        
    def testDeleteTaskWithCategory(self):
        self.category.addCategorizable(self.task1)
        self.task1.addCategory(self.category)
        self.delete('all')
        self.assertDoUndoRedo(lambda: self.failIf(self.category.categorizables()), 
            lambda: self.assertEqual(set([self.task1]), self.category.categorizables()))
        
    def testDeleteTaskWithTwoCategories(self):
        cat1 = category.Category('category 1')
        cat2 = category.Category('category 2')
        self.categories.extend([cat1, cat2])
        for cat in cat1, cat2:
            cat.addCategorizable(self.task1)
            self.task1.addCategory(cat)
        self.delete('all')
        self.assertDoUndoRedo(lambda: self.failIf(cat1.categorizables() or cat2.categorizables()), 
            lambda: self.failUnless(set([self.task1]) == cat1.categorizables() == cat2.categorizables()))
        
    def testDeleteTaskThatIsPrerequisite(self):
        self.task2.addPrerequisites([self.task1])
        self.task1.addDependencies([self.task2])
        self.taskList.append(self.task2)
        self.delete([self.task1])
        self.assertDoUndoRedo(lambda: self.failIf(self.task2.prerequisites()),
                              lambda: self.failUnless(self.task2.prerequisites()))

    def testDeleteTaskThatIsDependency(self):
        self.task2.addPrerequisites([self.task1])
        self.taskList.append(self.task2)
        self.delete([self.task2])
        self.assertDoUndoRedo(lambda: self.failIf(self.task1.dependencies()),
                              lambda: self.failUnless(self.task1.dependencies()))


class DeleteCommandWithTasksWithChildrenTest(CommandWithChildrenTestCase):
    def assertDeleteWorks(self):
        self.assertDoUndoRedo(self.assertParentAndAllChildrenDeleted,
            self.assertTaskListUnchanged)

    def assertParentAndAllChildrenDeleted(self):
        self.assertTaskList([self.task1])

    def assertTaskListUnchanged(self):
        self.assertTaskList(self.originalList)
        self.failUnlessParentAndChild(self.parent, self.child)
        self.failUnlessParentAndChild(self.child, self.grandchild)

    def assertShadowed(self, *shadowedTasks):
        for shadowedTask in shadowedTasks:
            self.failUnless(shadowedTask.isDeleted())
        
    def testDeleteParent(self):
        self.delete([self.parent])
        self.assertDeleteWorks()
        
    def testDeleteParentWhileShadowing(self):
        self.delete([self.parent], shadow=True)
        self.assertShadowed(self.parent, *self.parent.children())
        
    def testDeleteParentAndChild(self):
        self.delete([self.parent, self.child])
        self.assertDeleteWorks()

    def testDeleteParentAndGrandchild(self):
        self.delete([self.parent, self.grandchild])
        self.assertDeleteWorks()

    def testDeleteLastNotCompletedChildMarksParentAsCompleted(self):
        self.settings.setboolean('behavior', 
                                 'markparentcompletedwhenallchildrencompleted', 
                                 True)
        self.markCompleted([self.child2])
        self.delete([self.child])
        self.assertDoUndoRedo(
            lambda: self.failUnless(self.parent.completed()), 
            lambda: self.failIf(self.parent.completed()))
        
    def testDeleteParentAndChildWhenChildBelongsToCategory(self):
        self.category.addCategorizable(self.child)
        self.child.addCategory(self.category)
        self.delete([self.parent])
        self.assertDoUndoRedo(lambda: self.failIf(self.category.categorizables()), 
            lambda: self.assertEqual(set([self.child]), self.category.categorizables()))

    def testDeleteParentAndChildWhenParentAndChildBelongToDifferentCategories(self):
        cat1 = category.Category('category 1')
        cat2 = category.Category('category 2')
        self.categories.extend([cat1, cat2])
        cat1.addCategorizable(self.child)
        self.child.addCategory(cat1)
        cat2.addCategorizable(self.parent)
        self.parent.addCategory(cat2)
        self.delete([self.parent])
        self.assertDoUndoRedo(lambda: self.failIf(cat1.categorizables() or cat2.categorizables()), 
            lambda: self.failUnless(set([self.child]) == cat1.categorizables() and \
                                    set([self.parent]) == cat2.categorizables()))

    def testDeleteParentAndChildWhenParentAndChildBelongToSameCategory(self):
        for eachTask in self.parent, self.child:
            self.category.addCategorizable(eachTask)
            eachTask.addCategory(self.category)
        self.delete([self.parent])
        self.assertDoUndoRedo( \
            lambda: self.failIf(self.category.categorizables()), 
            lambda: self.assertEqualLists([self.parent, self.child], 
                                          self.category.categorizables()))


class DeleteCommandWithTasksWithEffortTest(CommandWithEffortTestCase):
    def testDeleteActiveTask(self):
        self.list = self.taskList
        self.delete([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(1, len(self.effortList)),
            lambda: self.assertEqual(2, len(self.effortList)))


class NewTaskCommandTest(TaskCommandTestCase):
    def new(self, **kwargs):
        newTaskCommand = command.NewTaskCommand(self.taskList, **kwargs)
        newTask = newTaskCommand.items[0]
        newTaskCommand.do()
        return newTask
    
    def testNewTask(self):
        newTask = self.new()
        self.assertDoUndoRedo(
            lambda: self.assertTaskList([self.task1, newTask]),
            lambda: self.assertTaskList(self.originalList))
        
    def testNewTaskWithCategory(self):
        cat = category.Category('cat')
        newTask = self.new(categories=[cat])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(set([cat]), newTask.categories()),
            lambda: self.assertTaskList(self.originalList))

    def testNewTaskWithCategory_AddsTaskToCategory(self):
        cat = category.Category('cat')
        newTask = self.new(categories=[cat])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(set([newTask]), cat.categorizables()),
            lambda: self.failIf(cat.categorizables()))

    def testNewTaskWithPrerequisite(self):
        newTask = self.new(prerequisites=[self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(set([self.task1]), newTask.prerequisites()),
            lambda: self.assertTaskList(self.originalList))
        
    def testNewTaskWithPrerequisite_AddsDependency(self):
        newTask = self.new(prerequisites=[self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(set([newTask]), self.task1.dependencies()),
            lambda: self.failIf(self.task1.dependencies()))

    def testNewTaskWithDependency(self):
        newTask = self.new(dependencies=[self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(set([self.task1]), newTask.dependencies()),
            lambda: self.assertTaskList(self.originalList))

    def testNewTaskWithDependency_AddsPrerequisite(self):
        newTask = self.new(dependencies=[self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(set([newTask]), self.task1.prerequisites()),
            lambda: self.failIf(self.task1.prerequisites()))
        
    def testNewTaskWithAttachment(self):
        att = attachment.FileAttachment('filename')
        newTask = self.new(attachments=[att])
        self.assertDoUndoRedo(
            lambda: self.assertEqual([att], newTask.attachments()),
            lambda: self.assertTaskList(self.originalList))
        
    def testNewTaskWithKeywords(self):
        dateTime = date.DateTime(2042, 2, 3)
        newTask = self.new(plannedStartDateTime=dateTime)
        self.assertEqual(dateTime, newTask.plannedStartDateTime())
        
    def testItemsAreNew(self):
        self.failUnless(command.NewTaskCommand(self.taskList).items_are_new())


class NewSubTaskCommandTest(TaskCommandTestCase):
    def testNewSubTask_WithoutSelection(self):
        self.newSubTask()
        self.assertDoUndoRedo(lambda: self.assertTaskList(self.originalList))

    def testNewSubTask(self):
        self.newSubTask([self.task1])
        newSubTask = self.task1.children()[0]
        self.assertDoUndoRedo(lambda: self.assertNewSubTask(newSubTask),
            lambda: self.assertTaskList(self.originalList))

    def assertNewSubTask(self, newSubTask):
        self.assertEqual(len(self.originalList) + 1, len(self.taskList))
        self.assertEqualLists([newSubTask], self.task1.children())

    def testNewSubTask_MarksParentAsNotCompleted(self):
        self.markCompleted([self.task1])
        self.newSubTask([self.task1])
        self.assertDoUndoRedo(lambda: self.failIf(self.task1.completed()),
            lambda: self.failUnless(self.task1.completed()))

    def testNewSubTask_MarksGrandParentAsNotCompleted(self):
        self.newSubTask([self.task1])
        self.markCompleted([self.task1])
        self.newSubTask([self.task1.children()[0]])
        self.assertDoUndoRedo(lambda: self.failIf(self.task1.completed()),
            lambda: self.failUnless(self.task1.completed()))

    def testNewCompletedSubTask(self):
        self.settings.setboolean('behavior', 
                                 'markparentcompletedwhenallchildrencompleted', 
                                 True)
        self.newSubTask([self.task1], markCompleted=True)
        self.assertDoUndoRedo(lambda: self.failUnless(self.task1.completed()),
            lambda: self.failIf(self.task1.completed()))
        
    def testNewSubTaskWithoutDueDateDoesntResetParentsDueDate(self):
        dueDateTime = date.Now() + date.TWO_HOURS
        self.task1.setDueDateTime(dueDateTime)
        self.newSubTask([self.task1])
        self.assertDoUndoRedo(lambda: self.assertEqual(dueDateTime, 
                                      self.task1.dueDateTime()))

    def testItemsAreNew(self):
        self.failUnless(command.NewSubTaskCommand(self.taskList, 
                                                  []).items_are_new())


class MarkCompletedCommandTest(CommandWithChildrenTestCase):
    def testMarkCompleted(self):
        self.markCompleted([self.task1])
        self.assertDoUndoRedo(
            lambda: self.failUnless(self.task1.completed()),
            lambda: self.failIf(self.task1.completed()))

    def testMarkCompleted_TaskAlreadyCompleted(self):
        self.task1.setCompletionDateTime()
        self.markCompleted([self.task1])
        self.assertDoUndoRedo(
            lambda: self.failUnless(self.task1.completed()))

    def testMarkCompletedParent(self):
        self.markCompleted([self.parent])
        self.assertDoUndoRedo(lambda: self.failUnless(self.child.completed()
            and self.child2.completed() and self.grandchild.completed()),
            lambda: self.failIf(self.child.completed() or
            self.child2.completed() or self.grandchild.completed()))

    def testMarkCompletedParent_WhenChildAlreadyCompleted(self):
        self.markCompleted([self.child])
        self.markCompleted([self.parent])
        self.assertDoUndoRedo(lambda: self.failUnless(self.child.completed()))

    def testMarkCompletedGrandChild(self):
        self.settings.setboolean('behavior', 
                                 'markparentcompletedwhenallchildrencompleted', 
                                 True)
        self.markCompleted([self.grandchild])
        self.assertDoUndoRedo(
            lambda: self.failUnless(self.child.completed() and 
                not self.parent.completed()), 
            lambda: self.failIf(self.child.completed() or 
                self.parent.completed()))

    def testMarkCompletedStopsEffortTracking(self):
        self.task1.addEffort(effort.Effort(self.task1))
        self.markCompleted([self.task1])
        self.assertDoUndoRedo(lambda: self.failIf(self.task1.isBeingTracked()), 
            lambda: self.failUnless(self.task1.isBeingTracked()))
            
    def testMarkCompletedChildDoesNotStopEffortTrackingOfParent(self):
        self.parent.addEffort(effort.Effort(self.parent))
        self.markCompleted([self.child])
        self.assertDoUndoRedo(lambda: self.failUnless(self.parent.isBeingTracked()))
        
    def testMarkRecurringTaskCompleted_CompletionDateIsNotSet(self):
        self.task1.setRecurrence(date.Recurrence('weekly'))
        self.markCompleted([self.task1])
        self.assertDoUndoRedo(lambda: self.failIf(self.task1.completed()))

    def testMarkRecurringTaskCompleted_PlannedStartDateIsIncreased(self):
        self.task1.setRecurrence(date.Recurrence('weekly'))
        plannedStartDateTime = self.task1.plannedStartDateTime()
        newPlannedStartDateTime = plannedStartDateTime + date.TimeDelta(days=7)
        self.markCompleted([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(newPlannedStartDateTime, self.task1.plannedStartDateTime()),
            lambda: self.assertEqual(plannedStartDateTime, self.task1.plannedStartDateTime()))

    def testMarkRecurringTaskCompleted_DueDateIsIncreased(self):
        self.task1.setRecurrence(date.Recurrence('weekly'))
        tomorrow = date.Tomorrow()
        self.task1.setDueDateTime(tomorrow)
        newDueDate = tomorrow + date.ONE_WEEK
        self.markCompleted([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(newDueDate, self.task1.dueDateTime()),
            lambda: self.assertEqual(tomorrow, self.task1.dueDateTime()))
        
    def testMarkRecurringTaskCompleted_ActualStartDateIsReset(self):
        self.task1.setRecurrence(date.Recurrence('weekly'))
        now = date.Now()
        self.task1.setActualStartDateTime(now)
        self.markCompleted([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(date.DateTime(), self.task1.actualStartDateTime()),
            lambda: self.assertEqual(now, self.task1.actualStartDateTime()))
        
    def testMarkParentWithRecurringChildCompleted_RemovesChildRecurrence(self):
        self.child.setRecurrence(date.Recurrence('daily'))
        self.markCompleted([self.parent])
        self.assertDoUndoRedo(
            lambda: self.failIf(self.child.recurrence()),
            lambda: self.assertEqual(date.Recurrence('daily'), self.child.recurrence()))

    def testMarkParentWithRecurringChildCompleted_MakesChildCompleted(self):
        self.child.setRecurrence(date.Recurrence('daily'))
        self.markCompleted([self.parent])
        self.assertDoUndoRedo(
            lambda: self.failUnless(self.child.completed()),
            lambda: self.failIf(self.child.completed()))


class EditPercentageCompleteTest(TaskCommandTestCase):
    def testEditPercentageComplete(self):
        self.editPercentageComplete([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(50, self.task1.percentageComplete()),
            lambda: self.assertEqual(0, self.task1.percentageComplete()))
        
    def testTaskIsStartedAfterEditingPercentageComplete(self):
        self.editPercentageComplete([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertNotEqual(date.DateTime(), self.task1.actualStartDateTime()),
            lambda: self.assertEqual(date.DateTime(), self.task1.actualStartDateTime()))


class MarkActiveCommandTest(TaskCommandTestCase):
    def testMarkInactiveTaskActive(self):
        self.markActive([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertNotEqual(date.DateTime(), self.task1.actualStartDateTime()),
            lambda: self.assertEqual(date.DateTime(), self.task1.actualStartDateTime()))
    
    def testMarkCompletedTaskActive(self):
        now = date.Now()
        self.task1.setCompletionDateTime(now)        
        self.markActive([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(date.DateTime(), self.task1.completionDateTime()),
            lambda: self.assertEqual(now, self.task1.completionDateTime()))

    def testIgnoreTaskThatIsAlreadyActive(self):
        now = date.Now()
        self.task1.setActualStartDateTime(now)
        self.markActive([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(now, self.task1.actualStartDateTime()))
        
    def testTaskWithFutureActualStartDateTime(self):
        tomorrow = date.Tomorrow()
        self.task1.setActualStartDateTime(tomorrow)
        self.markActive([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertAlmostEqual(date.Now().toordinal(), self.task1.actualStartDateTime().toordinal(), places=2),
            lambda: self.assertEqual(tomorrow, self.task1.actualStartDateTime()))


class MarkInactiveCommandTest(TaskCommandTestCase):
    def testMarkActiveTaskInactive(self):
        now = date.Now()
        self.task1.setActualStartDateTime(now)
        self.markInactive([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(date.DateTime(), self.task1.actualStartDateTime()),
            lambda: self.assertEqual(now, self.task1.actualStartDateTime()))
    
    def testMarkCompletedTaskInactive(self):
        now = date.Now()
        self.task1.setCompletionDateTime(now)
        self.markInactive([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(date.DateTime(), self.task1.completionDateTime()),
            lambda: self.assertEqual(now, self.task1.completionDateTime()))
        
    def testIgnoreTaskThatIsAlreadyInactive(self):
        self.markInactive([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(date.DateTime(), self.task1.actualStartDateTime()))
        
    def testTaskWithFutureActualStartDateTime(self):
        tomorrow = date.Tomorrow()
        self.task1.setActualStartDateTime(tomorrow)
        self.markInactive([self.task1])
        self.assertDoUndoRedo(
            lambda: self.assertEqual(date.DateTime(), self.task1.actualStartDateTime()),
            lambda: self.assertEqual(tomorrow, self.task1.actualStartDateTime()))
           

class DragAndDropTaskCommandTest(CommandWithChildrenTestCase):
    def testCannotDropOnParent(self):
        self.dragAndDrop([self.parent], [self.child])
        self.failIf(patterns.CommandHistory().hasHistory())
        
    def testCannotDropOnChild(self):
        self.dragAndDrop([self.child], [self.parent])
        self.failIf(patterns.CommandHistory().hasHistory())
        
    def testCannotDropOnGrandchild(self):
        self.dragAndDrop([self.grandchild], [self.parent])
        self.failIf(patterns.CommandHistory().hasHistory())

    def testDropAsRootTask(self):
        self.dragAndDrop([], [self.grandchild])
        self.assertDoUndoRedo(lambda: self.assertEqual(None, 
            self.grandchild.parent()), lambda:
            self.assertEqual(self.child, self.grandchild.parent()))
        

class PriorityCommandTestCase(TaskCommandTestCase):
    def setUp(self):
        super(PriorityCommandTestCase, self).setUp()
        self.taskList.append(self.task2)

    def assertDoUndoRedo(self, priority1do, priority2do, priority1undo, priority2undo):  # pylint: disable=W0221
        super(PriorityCommandTestCase, self).assertDoUndoRedo(
            lambda: self.failUnless(priority1do == self.task1.priority() and
                    priority2do == self.task2.priority()),
            lambda: self.failUnless(priority1undo == self.task1.priority() and
                    priority2undo == self.task2.priority()))

        
class MaxPriorityCommandTest(PriorityCommandTestCase):
    def maxPriority(self, tasks=None):
        command.MaxPriorityCommand(self.taskList, tasks or []).do()        
        
    def testEmptySelection(self):
        self.maxPriority()
        self.assertDoUndoRedo(0, 0, 0, 0)
        
    def testOneTaskWhenBothTasksHaveSamePriority(self):
        self.maxPriority([self.task1])
        self.assertDoUndoRedo(1, 0, 0, 0)
        
    def testBothTasksWhenBothTasksHaveSamePriority(self):
        self.maxPriority([self.task1, self.task2])
        self.assertDoUndoRedo(1, 1, 0, 0)
        
    def testMakeLowestPriorityTheMaxPriority(self):
        self.task2.setPriority(2)
        self.maxPriority([self.task1])
        self.assertDoUndoRedo(3, 2, 0, 2)


class MinPriorityCommandTest(PriorityCommandTestCase):        
    def minPriority(self, tasks=None):
        command.MinPriorityCommand(self.taskList, tasks or []).do()        

    def testEmptySelection(self):
        self.minPriority()
        self.assertDoUndoRedo(0, 0, 0, 0)
        
    def testOneTaskWhenBothTasksHaveSamePriority(self):
        self.minPriority([self.task1])
        self.assertDoUndoRedo(-1, 0, 0, 0)
        
    def testBothTasksWhenBothTasksHaveSamePriority(self):
        self.minPriority([self.task1, self.task2])
        self.assertDoUndoRedo(-1, -1, 0, 0)
        
    def testMakeLowestPriorityTheMaxPriority(self):
        self.task2.setPriority(-2)
        self.minPriority([self.task1])
        self.assertDoUndoRedo(-3, -2, 0, -2)


class IncreasePriorityCommandTest(PriorityCommandTestCase):
    def incPriority(self, tasks=None):
        command.IncPriorityCommand(self.taskList, tasks or []).do()
        
    def testEmptySelection(self):
        self.incPriority()
        self.assertDoUndoRedo(0, 0, 0, 0)
        
    def testOneTaskWhenBothTasksHaveSamePriority(self):
        self.incPriority([self.task1])
        self.assertDoUndoRedo(1, 0, 0, 0)
        
    def testBothTasksWhenBothTasksHaveSamePriority(self):
        self.incPriority([self.task1, self.task2])
        self.assertDoUndoRedo(1, 1, 0, 0)
        
    def testIncLowestPriority(self):
        self.task2.setPriority(-2)
        self.incPriority([self.task2])
        self.assertDoUndoRedo(0, -1, 0, -2)
    

class DecreasePriorityCommandTest(PriorityCommandTestCase):
    def decPriority(self, tasks=None):
        command.DecPriorityCommand(self.taskList, tasks or []).do()
        
    def testEmptySelection(self):
        self.decPriority()
        self.assertDoUndoRedo(0, 0, 0, 0)
        
    def testOneTaskWhenBothTasksHaveSamePriority(self):
        self.decPriority([self.task1])
        self.assertDoUndoRedo(-1, 0, 0, 0)
        
    def testBothTasksWhenBothTasksHaveSamePriority(self):
        self.decPriority([self.task1, self.task2])
        self.assertDoUndoRedo(-1, -1, 0, 0)
        
    def testDecLowestPriority(self):
        self.task2.setPriority(-2)
        self.decPriority([self.task2])
        self.assertDoUndoRedo(0, -3, 0, -2)


class EditReminderCommandTest(TaskCommandTestCase):
    def editReminder(self, tasks=None):
        command.EditReminderDateTimeCommand(self.taskList, tasks or [], 
                                            newValue=date.Now() + date.ONE_HOUR).do()
        
    def testEditReminder(self):
        self.editReminder([self.task1])
        self.assertDoUndoRedo(lambda: self.failUnless(self.task1.reminder()),
                              lambda: self.failIf(self.task1.reminder()))
        

class AddNoteCommandTest(TaskCommandTestCase):
    def addNote(self, tasks=None):
        command.AddNoteCommand(self.taskList, tasks or []).do()
    
    # pylint: disable=E1101
    
    def testEmptySelection(self):
        self.addNote()
        self.assertDoUndoRedo(lambda: self.failIf(self.task1.notes()))
        
    def testAddNote(self):
        self.addNote([self.task1])
        self.assertDoUndoRedo(lambda: self.failUnless(self.task1.notes()),
            lambda: self.failIf(self.task1.notes()))
            
    def testAddNoteToMultipleTasksAtOnce(self):
        self.addNote([self.task1, self.task2])
        self.assertDoUndoRedo(\
            lambda: self.failIf(self.task1.notes()[0] == self.task2.notes()[0]),
            lambda: self.failIf(self.task1.notes() or self.task2.notes()))
    

class EditDuePlannedStartDateCommandTest(TaskCommandTestCase):
    def testSetPlannedStartDateToTomorrow(self):
        previousStart = self.task1.plannedStartDateTime()
        newStart = date.Now() + date.ONE_HOUR
        self.editPlannedStart(newStart, [self.task1])
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(newStart, self.task1.plannedStartDateTime()),
            lambda: self.assertEqual(previousStart, self.task1.plannedStartDateTime()))

    def testSetDueDateToTomorrow(self):
        previousDue = self.task1.dueDateTime()
        newDue = date.Now() + date.ONE_HOUR
        self.editDue(newDue, [self.task1])
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(newDue, self.task1.dueDateTime()),
            lambda: self.assertEqual(previousDue, self.task1.dueDateTime()))

    def testPushingBackPlannedStartDatePushesBackDueDate(self):
        self.task1.setDueDateTime(date.Now() + date.TWO_HOURS) 
        previousPlannedStart = self.task1.plannedStartDateTime()
        previousDue = self.task1.dueDateTime()
        pushBack = date.ONE_HOUR
        newPlannedStart = previousPlannedStart + pushBack
        expectedDue = previousDue + pushBack
        self.editPlannedStart(newPlannedStart, [self.task1], keep_delta=True)
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(expectedDue, self.task1.dueDateTime()),
            lambda: self.assertEqual(previousDue, self.task1.dueDateTime()))

    def testPushingBackDueDatePushesBackPlannedStartDate(self):
        self.task1.setDueDateTime(date.Now() + date.TWO_HOURS)
        previousPlannedStart = self.task1.plannedStartDateTime()
        previousDue = self.task1.dueDateTime()
        pushBack = date.ONE_HOUR
        newDue = previousDue + pushBack
        expectedPlannedStart = previousPlannedStart + pushBack
        self.editDue(newDue, [self.task1], keep_delta=True)
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(expectedPlannedStart, self.task1.plannedStartDateTime()),
            lambda: self.assertEqual(previousPlannedStart, self.task1.plannedStartDateTime()))

    def testPushingBackPlannedStartDateDoesNotPushBackDueDate(self):
        self.task1.setDueDateTime(date.Now() + date.TWO_HOURS) 
        previousPlannedStart = self.task1.plannedStartDateTime()
        previousDue = self.task1.dueDateTime()
        pushBack = date.ONE_HOUR
        newPlannedStart = previousPlannedStart + pushBack
        expectedDue = previousDue
        self.editPlannedStart(newPlannedStart, [self.task1])
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(expectedDue, self.task1.dueDateTime()),
            lambda: self.assertEqual(previousDue, self.task1.dueDateTime()))

    def testPushingBackDueDateDoesNotPushBackPlannedStartDate(self):
        self.task1.setDueDateTime(date.Now() + date.TWO_HOURS)
        previousPlannedStart = self.task1.plannedStartDateTime()
        previousDue = self.task1.dueDateTime()
        pushBack = date.ONE_HOUR
        newDue = previousDue + pushBack
        expectedPlannedStart = previousPlannedStart
        self.editDue(newDue, [self.task1])
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(expectedPlannedStart, self.task1.plannedStartDateTime()),
            lambda: self.assertEqual(previousPlannedStart, self.task1.plannedStartDateTime()))

    def testMissingDueDateIsNotPushedBack(self):
        previousPlannedStart = self.task1.plannedStartDateTime()
        pushBack = date.ONE_HOUR
        newPlannedStart = previousPlannedStart + pushBack
        expectedDue = date.DateTime()
        self.task1.setDueDateTime(expectedDue)
        self.editPlannedStart(newPlannedStart, [self.task1], keep_delta=True)
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(expectedDue, self.task1.dueDateTime()))

    def testMissingPlannedStartDateIsNotPushedBack(self):
        self.task1.setPlannedStartDateTime(date.DateTime())
        self.task1.setDueDateTime(date.Now() + date.TWO_HOURS)
        previousDue = self.task1.dueDateTime()
        pushBack = date.ONE_HOUR
        newDue = previousDue + pushBack
        expectedStart = date.DateTime()
        self.editDue(newDue, [self.task1], keep_delta=True)
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(expectedStart, self.task1.plannedStartDateTime()))

    def testDueDateIsNotPushedBackWhenPlannedStartDateIsMissing(self):
        self.task1.setPlannedStartDateTime(date.DateTime())
        self.task1.setDueDateTime(date.Now() + date.TWO_HOURS)
        pushBack = date.ONE_HOUR
        newStart = date.Now() + pushBack
        expectedDue = self.task1.dueDateTime()
        self.editPlannedStart(newStart, [self.task1], keep_delta=True)
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(expectedDue, self.task1.dueDateTime()))

    def testPlannedStartDateIsNotPushedBackWhenDueDateIsMissing(self):
        pushBack = date.ONE_HOUR
        newDue = date.Now() + pushBack
        expectedStart = self.task1.plannedStartDateTime()
        self.task1.setDueDateTime(date.DateTime())
        self.editDue(newDue, [self.task1], keep_delta=True)
        self.assertDoUndoRedo(\
            lambda: self.assertEqual(expectedStart, self.task1.plannedStartDateTime()))
        
    def testSetInfinitePlannedStartDate(self):
        newPlannedStartDateTime = date.DateTime()
        originalPlannedStartDateTime = self.task1.plannedStartDateTime()
        self.editPlannedStart(newPlannedStartDateTime, [self.task1])
        self.assertDoUndoRedo(lambda: self.assertEqual(newPlannedStartDateTime, self.task1.plannedStartDateTime()),
                              lambda: self.assertEqual(originalPlannedStartDateTime, self.task1.plannedStartDateTime()))
        
    def testSetInfinitePlannedStartDateDoesNotChangeDueDate(self):
        newPlannedStartDateTime = date.DateTime()
        originalDueDateTime = self.task1.dueDateTime()
        self.editPlannedStart(newPlannedStartDateTime, [self.task1], keep_delta=True)
        self.assertDoUndoRedo(lambda: self.assertEqual(originalDueDateTime, 
                                                       self.task1.dueDateTime()))
        
    def testSetInfiniteDueDate(self):
        newDueDateTime = date.DateTime()
        originalDueDateTime = self.task1.dueDateTime()
        self.editDue(newDueDateTime, [self.task1])
        self.assertDoUndoRedo(lambda: self.assertEqual(newDueDateTime, self.task1.dueDateTime()),
                              lambda: self.assertEqual(originalDueDateTime, self.task1.dueDateTime()))
        
    def testSetInfiniteDueDateDoesNotChangePlannedStartDate(self):
        newDueDateTime = date.DateTime()
        originalPlannedStartDateTime = self.task1.plannedStartDateTime()
        self.editDue(newDueDateTime, [self.task1], keep_delta=True)
        self.assertDoUndoRedo(lambda: self.assertEqual(originalPlannedStartDateTime, 
                                                       self.task1.plannedStartDateTime()))



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from buildlib import clean, bdist_rpm_fedora, bdist_rpm_opensuse, bdist_deb, \
    bdist_winpenpack, bdist_portableapps
from setup import setupOptions
from taskcoachlib import meta
import sys
import os
import glob
import wx


setupOptions['cmdclass'] = dict(clean=clean,
                                bdist_rpm_fedora=bdist_rpm_fedora,
                                bdist_rpm_opensuse=bdist_rpm_opensuse,
                                bdist_deb=bdist_deb,
                                bdist_winpenpack=bdist_winpenpack,
                                bdist_portableapps=bdist_portableapps)
                                
distdir = 'dist'
builddir = 'build'


manifest = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<assembly xmlns="urn:schemas-microsoft-com:asm.v1" manifestVersion="1.0">
    <assemblyIdentity version="0.64.1.0" processorArchitecture="x86" 
    name="Controls" type="win32"/>
    <description>%s</description>
    <dependency>
        <dependentAssembly>
            <assemblyIdentity type="win32" 
            name="Microsoft.Windows.Common-Controls" version="6.0.0.0" 
            processorArchitecture="X86" publicKeyToken="6595b64144ccf1df"
            language="*"/>
        </dependentAssembly>
    </dependency>
    <dependency>
        <dependentAssembly>
            <assemblyIdentity type="win32" 
            name="Microsoft.VC90.CRT" version="9.0.21022.8"
            processorArchitecture="x86" publicKeyToken="1fc8b3b9a1e18e3b" />
        </dependentAssembly>
    </dependency>
</assembly>
""" % meta.name

doubleline = '================================================================\n'

header = doubleline + '%(name)s - %(description)s\n' % meta.metaDict + doubleline

aboutText = header + '''
Version %(version)s, %(date)s

By %(author)s <%(author_email)s>

%(url)s

%(copyright)s
%(license)s

''' % meta.metaDict + doubleline

installText = header + '''

--- Prerequisites ----------------------------------------------

You need Python version %(pythonversion)s or higher, wxPython 
version %(wxpythonversion)s or higher, and Twisted version
%(twistedversionnumber)s or higher.


--- Testing ----------------------------------------------------

Before installing, you may want to run the unittests included.
Issue the following command:

  cd tests; python test.py

If all goes well, you should see a number of dots appearing and
the message 'Ran X tests in Y seconds. OK'. If not, you'll get
one or more failed tests. In that case, please run the tests
again, redirecting the output to a textfile, like this:

  python test.py 2> errors.txt

Please mail me the errors.txt file and your platform information
(operating system version, Python version and wxPython version).


--- Installation -----------------------------------------------

There are two options to install %(name)s: 

First, you can simply move this directory to some suitable 
location and run taskcoach.py (or taskcoach.pyw if you are on 
the Windows platform) from there.

Alternatively, you can use the Python distutils setup script
to let Python install %(name)s for you. In that case run the
following command:

  python setup.py install

If you have a previous version of %(name)s installed, you may
need to force old files to be overwritten, like this:

  python setup.py install --force

''' % meta.metaDict + doubleline

buildText = header + '''

--- Building ---------------------------------------------------

To be done.

''' % meta.metaDict + doubleline


def writeFile(filename, text, directory='.'):  # pylint: disable=W0621
    if not os.path.exists(directory):
        os.mkdir(directory)
    with file(os.path.join(directory, filename), 'w') as textFile:
        textFile.write(text)


def createDocumentation():
    writeFile('README.txt', aboutText)
    writeFile('INSTALL.txt', installText)
    writeFile('LICENSE.txt', meta.licenseText)


def createInnoSetupScript():
    script = file('build.in/windows/taskcoach.iss').read()
    writeFile('taskcoach.iss', script % meta.metaDict, builddir)


def createDebianChangelog():
    changelog = file('build.in/debian/changelog').read()
    writeFile('changelog', changelog % meta.metaDict, 
              os.path.join(builddir, 'debian'))


if sys.argv[1] == 'py2exe':
    from distutils.core import setup
    import py2exe  # pylint: disable=F0401
    import shutil
    shutil.copyfile('taskcoach.pyw', 'taskcoach_console.py')
    py2exeDistdir = '%s-%s-win32exe' % (meta.filename, meta.version)
    # Get .mo files for wxWidgets:
    locale_dir = os.path.join(os.path.dirname(wx.__file__), 'locale')
    mo_path = os.path.join('LC_MESSAGES', 'wxstd.mo')
    mo_files = []
    for language_dir in os.listdir(locale_dir):
        mo_abs_filename = os.path.join(locale_dir, language_dir, mo_path)
        mo_rel_dir = os.path.join('locale', language_dir, 'LC_MESSAGES')
        mo_files.append((mo_rel_dir, [mo_abs_filename]))
    # DLL's we redistribute so people don't have to download them:
    dll_files = [('', ['dist.in/gdiplus.dll']), ('Microsoft.VC90.CRT',
                                                 ['dist.in/msvcp90.dll',
                                                  'dist.in/msvcr90.dll',
                                                  'dist.in/Microsoft.VC90.CRT.manifest'])]
    setupOptions.update({
        'windows': [{'script': 'taskcoach.pyw', 
            'other_resources': [(24, 1, manifest)],
            'icon_resources': [(1, 'icons.in/taskcoach.ico')]}],
        'console': [{'script': 'taskcoach_console.py',
            'other_resources': [(24, 1, manifest)]}],
        'options': {'py2exe': {
            'compressed': 1, 
            'optimize': 2, 
            # We need to explicitly include these packages because they 
            # are imported implicitly:
            'packages': ['taskcoachlib.i18n',
                          'taskcoachlib.thirdparty.keyring',
                          'taskcoachlib.thirdparty.pubsub',
                          'taskcoachlib.thirdparty.pubsub.core',
                          'taskcoachlib.thirdparty.pubsub.core.kwargs',
                          'taskcoachlib.thirdparty._weakrefset'], 
            'dist_dir': os.path.join(builddir, py2exeDistdir),
            'dll_excludes': ['MSVCR80.dll', 'UxTheme.dll']}},
        'data_files': dll_files + mo_files})
    os.environ['PATH'] = 'dist.in;' + os.environ['PATH']
 
elif sys.argv[1] == 'py2app':
    from setuptools import setup  # pylint: disable=W0404
    setupOptions.update(dict(app=['taskcoach.py'], 
        setup_requires=['py2app'],
        options=dict(py2app=dict(argv_emulation=True, compressed=True,
            dist_dir=os.path.join(builddir, 'Task Coach'), optimize=2, iconfile='icons.in/taskcoach.icns', 
            # We need to explicitly include i18n modules because they 
            # are imported implicitly via __import__:
            includes=[filename[:-3].replace('/', '.') for filename \
                      in glob.glob('taskcoachlib/i18n/*.py') + \
                      glob.glob('taskcoachlib/thirdparty/pubsub/*.py') + \
                      glob.glob('taskcoachlib/thirdparty/pubsub/core/*.py') + \
                      glob.glob('taskcoachlib/thirdparty/pubsub/core/kwargs/*.py')],
            plist=dict(CFBundleIconFile='taskcoach.icns',
                       CFBundleDocumentTypes=[dict(CFBundleTypeExtensions=['tsk'],
                                                   CFBundleTypeIconFile='taskcoach.icns',
                                                   CFBundleTypeName='%s task file' % meta.name,
                                                   CFBundleTypeRole='Editor')])))))
    
elif sys.argv[1] == 'bdist_rpm_fedora':
    from distutils.core import setup
    spec_file = file('build.in/fedora/taskcoach.spec').read() % meta.metaDict
    spec_file = spec_file.split('\n')
    setupOptions.update(dict(options=dict(bdist_rpm_fedora=dict(\
        spec_file=spec_file, icon='icons.in/taskcoach.png', 
        desktop_file='build.in/linux_common/taskcoach.desktop'))))
    # On Fedora, to keep the rpm build process going when it finds 
    # unpackaged files you need to create a ~/.rpmmacros file 
    # containing the line '%_unpackaged_files_terminate_build 0'.

elif sys.argv[1] == 'bdist_rpm_opensuse':
    from distutils.core import setup
    spec_file = file('build.in/opensuse/taskcoach.spec').read() % meta.metaDict
    spec_file = spec_file.split('\n')
    setupOptions.update(dict(options=dict(bdist_rpm_opensuse=dict(\
        spec_file=spec_file, icon='icons.in/taskcoach.png', 
        desktop_file='build.in/linux_common/taskcoach.desktop'))))

elif sys.argv[1] in ['bdist_deb', 'bdist_ubuntu']:
    from distutils.core import setup
    bdist_deb = dict(\
        package=meta.data.filename_lower, title=meta.data.name, 
        description=meta.data.description, 
        long_description=meta.data.long_description, 
        version=meta.data.version,
        author=meta.data.author, author_email=meta.data.author_email,
        copyright=meta.data.copyright,
        license=meta.data.license_title_and_version, 
        license_abbrev=meta.data.license_title_and_version_abbrev,
        license_path='/usr/share/common-licenses/GPL-3',
        license_summary=meta.data.license_notice, 
        wxpythonversion=meta.data.wxpythonversionnumber,
        twistedversion=meta.data.twistedversionnumber,
        subsection='Office', url=meta.data.url,
        command='/usr/bin/taskcoach.py',
        changelog_content=file('changelog_content', 'rb').read().rstrip(),
        sdist_exclude='buildlib')

    if sys.argv[1] == 'bdist_ubuntu':
        bdist_deb['distribution'] = sys.argv[2]
        bdist_deb['version'] = '%s-0ubuntu%s~%s' % (meta.version, sys.argv[3], sys.argv[2])
        bdist_deb['section'] = 'editors'
        sys.argv[1] = 'bdist_deb'
        del sys.argv[2]
        del sys.argv[2]

    setupOptions.update(dict(options=dict(bdist_deb=bdist_deb)))
    
elif sys.argv[1] == 'bdist_winpenpack':
    from distutils.core import setup
    setupOptions.update(dict(options=dict(bdist_winpenpack=dict(\
        version=meta.data.version,
        license=meta.data.license_title,
        url=meta.data.url,
        filename=meta.data.filename,
        date=meta.data.date))))
    
elif sys.argv[1] == 'bdist_portableapps':
    from distutils.core import setup
    setupOptions.update(dict(options=dict(bdist_portableapps=dict(\
        name=meta.data.name,
        version=meta.data.version,
        license=meta.data.license_title,
        url=meta.data.url,
        filename=meta.data.filename,
        date=meta.data.date))))
    
else:
    from distutils.core import setup


if __name__ == '__main__':
    for directory in builddir, distdir:
        if not os.path.exists(directory):
            os.mkdir(directory)
    createDocumentation()
    setup(**setupOptions)  # pylint: disable=W0142
    if sys.argv[1] == 'py2exe':
        createInnoSetupScript()
